# -*- coding: utf-8 -*-
"""Test Control Panel for plone.mls.core."""

from plone import api
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry import Registry
from ps.plone.mls.browser.interfaces import IPloneMLSLayer
from ps.plone.mls.interfaces import IMLSBaseSettings
from ps.plone.mls.interfaces import IMLSCachingSettings
from ps.plone.mls.interfaces import IMLSContactInfoSettings
from ps.plone.mls.interfaces import IMLSUISettings
from ps.plone.mls.testing import INTEGRATION_TESTING
from zope.component import getMultiAdapter
from zope.interface import alsoProvides


try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestMLSBaseControlPanel(unittest.TestCase):
    """Validate base control panel is available."""
    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']
        alsoProvides(self.portal.REQUEST, IPloneMLSLayer)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.registry = Registry()
        self.registry.registerInterface(IMLSBaseSettings)

    def test_controlpanel_view(self):
        """Validate that the configuration view is available."""
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='mls-controlpanel-base',
        )
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_protected(self):
        """Validate that the configuration view needs authentication."""
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(
            Unauthorized,
            self.portal.restrictedTraverse,
            '@@mls-controlpanel-base',
        )

    def test_mls_in_controlpanel(self):
        """Validate that there is an MLS entry in the control panel."""
        controlpanel = api.portal.get_tool(name='portal_controlpanel')
        actions = [
            a.getAction(self)['id'] for a in controlpanel.listActions()
        ]
        self.assertTrue('ps_plone_mls' in actions)


class TestMLSCachingControlPanel(unittest.TestCase):
    """Validate caching control panel is available."""
    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']
        alsoProvides(self.portal.REQUEST, IPloneMLSLayer)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.registry = Registry()
        self.registry.registerInterface(IMLSCachingSettings)

    def test_controlpanel_view(self):
        """Validate that the configuration view is available."""
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='mls-controlpanel-caching',
        )
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_protected(self):
        """Validate that the configuration view needs authentication."""
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(
            Unauthorized,
            self.portal.restrictedTraverse,
            '@@mls-controlpanel-caching',
        )


class TestMLSContactInfoControlPanel(unittest.TestCase):
    """Validate contact info control panel is available."""
    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']
        alsoProvides(self.portal.REQUEST, IPloneMLSLayer)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.registry = Registry()
        self.registry.registerInterface(IMLSContactInfoSettings)

    def test_controlpanel_view(self):
        """Validate that the configuration view is available."""
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='mls-controlpanel-contact-info',
        )
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_protected(self):
        """Validate that the configuration view needs authentication."""
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(
            Unauthorized,
            self.portal.restrictedTraverse,
            '@@mls-controlpanel-contact-info',
        )


class TestMLSUIControlPanel(unittest.TestCase):
    """Validate ui control panel is available."""
    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']
        alsoProvides(self.portal.REQUEST, IPloneMLSLayer)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.registry = Registry()
        self.registry.registerInterface(IMLSUISettings)

    def test_controlpanel_view(self):
        """Validate that the configuration view is available."""
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='mls-controlpanel-ui',
        )
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_protected(self):
        """Validate that the configuration view needs authentication."""
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(
            Unauthorized,
            self.portal.restrictedTraverse,
            '@@mls-controlpanel-ui',
        )


class TestMLSUsageControlPanel(unittest.TestCase):
    """Validate usage control panel is available."""
    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']
        alsoProvides(self.portal.REQUEST, IPloneMLSLayer)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_controlpanel_view(self):
        """Validate that the configuration view is available."""
        view = getMultiAdapter(
            (self.portal, self.portal.REQUEST),
            name='mls-controlpanel-usage',
        )
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_protected(self):
        """Validate that the configuration view needs authentication."""
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(
            Unauthorized,
            self.portal.restrictedTraverse,
            '@@mls-controlpanel-usage',
        )



# -*- coding: utf-8 -*-
"""Test utilities from ps.plone.mls.browser.utilities."""

from plone import api
from ps.plone.mls.testing import INTEGRATION_TESTING


try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestUtilitiesView(unittest.TestCase):
    """Validate the utility methods."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']
        self.view = self.portal.restrictedTraverse('@@psplonemls-utils')

    def test_smart_truncate(self):
        """Validate the 'smart_truncate' method."""
        TEXT = (
            u'Lorem ipsum dolor sit amet, consectetur adipiscing elit. '
            u'Mauris suscipit orci sed sollicitudin convallis. Vestibulum '
            u'fringilla pretium velit, et elementum sapien. Pellentesque '
            u'habitant morbi tristique senectus et netus et malesuada fames '
            u'ac turpis egestas. Morbi neque quam, volutpat sed nulla non, '
            u'placerat volutpat purus. Etiam vitae nibh eget ipsum tincidunt '
            u'cursus rhoncus fermentum urna. Aliquam in ultrices magna. '
            u'Pellentesque habitant morbi tristique senectus et netus et '
            u'malesuada fames ac turpis egestas. Cras mollis lorem nibh, in '
            u'pharetra lorem tincidunt eget. Etiam ultrices vehicula tortor '
            u'ut porta. Quisque commodo ex non dapibus tincidunt.'
        )

        truncated = self.view.smart_truncate(None)
        self.assertIsNone(truncated)

        truncated = self.view.smart_truncate(TEXT)
        self.assertEqual(len(truncated), 654)

        api.portal.set_registry_record(
            name='plone.mls.listing.interfaces.IMLSUISettings.truncate_texts',
            value=True,
        )

        truncated = self.view.smart_truncate(TEXT)
        self.assertEqual(len(truncated), 319)

        self.assertEqual(
            truncated,
            u'Lorem ipsum dolor sit amet, consectetur adipiscing elit. '
            u'Mauris suscipit orci sed sollicitudin convallis. Vestibulum '
            u'fringilla pretium velit, et elementum sapien. Pellentesque '
            u'habitant morbi tristique senectus et netus et malesuada fames '
            u'ac turpis egestas. Morbi neque quam, volutpat sed nulla non, '
            u'placerat volutpat...',
        )

        api.portal.set_registry_record(
            name='plone.mls.listing.interfaces.IMLSUISettings.truncate_length',
            value=15,
        )

        truncated = self.view.smart_truncate(TEXT)
        self.assertEqual(len(truncated), 14)

        self.assertEqual(
            truncated,
            u'Lorem ipsum...',
        )



# -*- coding: utf-8 -*-
"""Featured Listings."""

from plone.app.textfield import RichText
from plone.autoform import directives
from plone.dexterity.content import Item
from plone.supermodel import model
from ps.plone.mls import _
from ps.plone.mls import interfaces
from ps.plone.mls import PLONE_4
from zope import schema
from zope.interface import implementer


if PLONE_4:
    from plone.app.textfield.widget import RichTextFieldWidget


class IFeaturedListings(model.Schema):
    """A list of featured MLS Listings."""

    listing_ids = schema.List(
        description=_(
            u'Add one Listing ID per line for each entry to show up.',
        ),
        title=_(u'MLS Listing IDs'),
        unique=True,
        value_type=schema.TextLine(
            title=_(u'ID'),
        ),
    )

    if PLONE_4:
        directives.widget(body_text=RichTextFieldWidget)
    body_text = RichText(
        required=False,
        title=_('Body Text'),
    )


@implementer(
    IFeaturedListings,
    interfaces.IListingTraversable,
)
class FeaturedListings(Item):
    """A list of featured MLS Listings."""



# -*- coding: utf-8 -*-
"""Test 'Featured Listings' Content Type."""

from plone import api
from plone.app.content.interfaces import INameFromTitle
from plone.app.dexterity.behaviors.exclfromnav import IExcludeFromNavigation
from plone.dexterity.interfaces import IDexterityFTI
from ps.plone.mls.content.featured import IFeaturedListings
from ps.plone.mls.testing import INTEGRATION_TESTING
from zope.component import createObject
from zope.component import queryUtility


try:
    import unittest2 as unittest
except ImportError:
    import unittest


CT = 'ps.plone.mls.featured'


class FeaturedListingsIntegrationTestCase(unittest.TestCase):
    """Test Case for the 'ps.plone.mls.featured' content type."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']

        with api.env.adopt_roles(['Manager']):
            self.folder = api.content.create(self.portal, 'Folder', 'folder')

        self.ct = api.content.create(self.folder, CT, 'ct1')

    def test_ct_available(self):
        """Validate that the content type is available."""
        portal_types = api.portal.get_tool(name='portal_types')
        self.assertTrue(CT in portal_types)

    def test_fti(self):
        """Validate that the factory type information is available."""
        fti = queryUtility(IDexterityFTI, name=CT)
        self.assertIsNotNone(fti)

    def test_adding(self):
        """Validate the created content type."""
        self.assertTrue(IFeaturedListings.providedBy(self.ct))

    def test_schema(self):
        """Validate the correct schema."""
        fti = queryUtility(IDexterityFTI, name=CT)
        schema = fti.lookupSchema()
        self.assertEqual(IFeaturedListings, schema)

    def test_factory(self):
        """Validate the content type factory."""
        fti = queryUtility(IDexterityFTI, name=CT)
        factory = fti.factory
        new_object = createObject(factory)
        self.assertTrue(IFeaturedListings.providedBy(new_object))

    def test_behaviors(self):
        """Validate that the required behaviors are available."""
        self.assertTrue(INameFromTitle.providedBy(self.ct))
        self.assertTrue(IExcludeFromNavigation.providedBy(self.ct))

    def test_selectable_as_folder_default_view(self):
        """Validate that we can set our content type as a default view."""
        self.folder.setDefaultPage('ct1')
        self.assertEqual(self.folder.getDefaultPage(), 'ct1')



# -*- coding: utf-8 -*-
"""Usage overview control panel."""

from plone import api
from plone.mls.core.interfaces import ILocalMLSSettings
from plone.mls.listing.browser.listing_collection import IListingCollection
from plone.mls.listing.browser.listing_search import IListingSearch
from plone.mls.listing.browser.recent_listings import IRecentListings
from plone.mls.listing.interfaces import ILocalAgencyInfo
from Products.Five.browser import BrowserView
from ps.plone.mls.browser.listings.featured import IFeaturedListings
from ps.plone.mls.interfaces import IDevelopmentCollection
from ps.plone.mls.interfaces import IListingSearchBanner


class UsageControlPanel(BrowserView):
    """MLS Embedding Usage overview control panel."""

    def get_listing_collections(self):
        """Get all activated listing collections."""
        return api.content.find(
            object_provides=IListingCollection,
            sort_on='sortable_title',
        )

    def get_listing_searches(self):
        """Get all activated listing searches."""
        return api.content.find(
            object_provides=IListingSearch,
            sort_on='sortable_title',
        )

    def get_listing_search_banners(self):
        """Get all activated listing search banners."""
        return api.content.find(
            object_provides=IListingSearchBanner,
            sort_on='sortable_title',
        )

    def get_recent_listings(self):
        """Get all activated recent listings."""
        return api.content.find(
            object_provides=IRecentListings,
            sort_on='sortable_title',
        )

    def get_featured_listings(self):
        """Get all activated featured listings."""
        return api.content.find(
            object_provides=IFeaturedListings,
            sort_on='sortable_title',
        )

    def get_development_collections(self):
        """Get all activated development collections."""
        return api.content.find(
            object_provides=IDevelopmentCollection,
            sort_on='sortable_title',
        )

    def get_local_mls_settings(self):
        """Get all activated local MLS settings."""
        return api.content.find(
            object_provides=ILocalMLSSettings,
            sort_on='sortable_title',
        )

    def get_local_contact_infos(self):
        """Get all activated local contact infos."""
        return api.content.find(
            object_provides=ILocalAgencyInfo,
            sort_on='sortable_title',
        )



# -*- coding: utf-8 -*-
"""Navigation Breadcrumb customizations."""

from Products.CMFPlone.browser.navigation import get_view_url
from Products.CMFPlone.browser.navigation import PhysicalNavigationBreadcrumbs
from ps.plone.mls import _
from zope.annotation.interfaces import IAnnotations


TRAVERSE_TITLES = {
    'listings': _(u'Listings'),
}


class DevelopmentDetailsNavigationBreadcrumbs(PhysicalNavigationBreadcrumbs):
    """Custom breadcrumb navigation for development details."""

    def breadcrumbs(self):
        base = super(
            DevelopmentDetailsNavigationBreadcrumbs,
            self,
        ).breadcrumbs()

        name, item_url = get_view_url(self.context)

        development_id = getattr(self.request, 'development_id', None)
        last_item = self.request.steps[-2:-1]
        if development_id is None or self.context.id not in last_item:
            return base

        cache = IAnnotations(self.request)
        item = cache.get('ps.plone.mls.development.traversed', None)
        if item is None:
            return base

        try:
            title = item.title.value
        except Exception:
            return base

        url = '/'.join([item_url, development_id])

        base += ({
            'absolute_url': url,
            'Title': title,
        },)

        listing_id = getattr(self.request, 'listing_id', None)
        if listing_id is not None:
            base += ({
                'absolute_url': '/'.join([url, listing_id]),
                'Title': listing_id.upper(),
            },)

        subpath = getattr(self.request, 'subpath', [])
        if subpath and len(subpath) == 1:
            item = subpath[0]
            title = TRAVERSE_TITLES.get(item, None)
            if title is not None:
                base += ({
                    'absolute_url': '/'.join([url, item]),
                    'Title': title,
                },)

        return base


class ListingDetailsNavigationBreadcrumbs(PhysicalNavigationBreadcrumbs):
    """Custom breadcrumb navigation for listing details."""

    def breadcrumbs(self):
        base = super(ListingDetailsNavigationBreadcrumbs, self).breadcrumbs()

        name, item_url = get_view_url(self.context)

        item_id = getattr(self.request, 'listing_id', None)
        last_item = self.request.steps[-2:-1]
        if item_id is not None and self.context.id in last_item:
            base += ({
                'absolute_url': '/'.join([item_url, item_id]),
                'Title': item_id.upper(),
            },)

        return base



# -*- coding: utf-8 -*-
"""Propertyshelf MLS Plone Embedding."""

from plone import api as ploneapi
from ps.plone.mls import config
from zope.i18nmessageid import MessageFactory

import logging


PLONE_4 = '4' <= ploneapi.env.plone_version() < '5'
PLONE_5 = '5' <= ploneapi.env.plone_version() < '6'

logger = logging.getLogger(config.PROJECT_NAME)
_ = MessageFactory('ps.plone.mls')



# -*- coding: utf-8 -*-
"""Browser utilities."""

from ps.plone.mls import utils
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.browser import BrowserView


class IUtilities(Interface):
    """Some common utilities for the MLS Embedding."""

    def smart_truncate(content):
        """Truncate a string for some max length but split at word boundary."""


@implementer(IUtilities)
class Utilities(BrowserView):
    """Some common utilities for the MLS Embedding."""

    def smart_truncate(self, content):
        return utils.smart_truncate(content)



# -*- coding: utf-8 -*-
"""Test Layer for ps.plone.mls."""

from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import Layer
from plone.testing import z2
from ps.plone.mls.tests import utils

import pkg_resources
import responses


try:
    pkg_resources.get_distribution('plone.app.contenttypes')
except pkg_resources.DistributionNotFound:
    HAS_PA_CONTENTTYPES = False
else:
    HAS_PA_CONTENTTYPES = True


class MLSAPIMockLayer(Layer):
    """Load test fixtures using responses to mock API requests."""

    def testSetUp(self):
        responses.start()
        utils.setup_plone_mls_fixtures()

    def testTearDown(self):
        try:
            responses.stop()
        except RuntimeError:
            pass
        finally:
            responses.reset()


MLSAPIMOCK = MLSAPIMockLayer()


class Fixture(PloneSandboxLayer):
    """Custom Test Layer for ps.plone.mls."""

    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        """Set up Zope for testing."""
        # Load ZCML
        import plone.app.dexterity
        self.loadZCML(package=plone.app.dexterity)

        if HAS_PA_CONTENTTYPES:
            import plone.app.contenttypes
            self.loadZCML(package=plone.app.contenttypes)

        import ps.plone.mls
        self.loadZCML(package=ps.plone.mls)

    def setUpPloneSite(self, portal):
        """Set up a Plone site for testing."""
        super(Fixture, self).setUpPloneSite(portal)
        # Install into Plone site using portal_setup
        # self.applyProfile(portal, 'Products.CMFPlone:plone')

        # Plone 5 support
        if HAS_PA_CONTENTTYPES:
            self.applyProfile(portal, 'plone.app.contenttypes:default')

        self.applyProfile(portal, 'ps.plone.mls:default')
        self.applyProfile(portal, 'ps.plone.mls:testfixture')
        portal.portal_workflow.setDefaultChain('simple_publication_workflow')


FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE, ),
    name='ps.plone.mls:Integration',
)

FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(MLSAPIMOCK, FIXTURE, z2.ZSERVER_FIXTURE),
    name='ps.plone.mls:Functional',
)


class FunctionalMLSAPIMockLayer(MLSAPIMockLayer):

    defaultBases = (
        FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE,
    )


ACCEPTANCE_TESTING = FunctionalMLSAPIMockLayer(
    name='ps.plone.mls:Acceptance')

ROBOT_TESTING = MLSAPIMockLayer(name='ps.plone.mls:Robot')



# -*- coding: utf-8 -*-
"""Interface definitions."""

from plone.directives import form
from plone.mls.core.interfaces import IMLSSettings as IMLSBaseSettings
from plone.mls.listing import interfaces
from ps.plone.mls import _
from zope import schema
from zope.interface import Interface


assert(IMLSBaseSettings)
IGlobalContactSettingsForm = interfaces.IMLSAgencyContactInfoSettingsEditForm
IMLSContactInfoSettings = interfaces.IMLSAgencyContactInformation
IMLSUISettings = interfaces.IMLSUISettings


class IListingTraversable(Interface):
    """Marker interface for traversable listings."""


class IDevelopmentTraversable(Interface):
    """Marker interface for traversable listings."""


class IBaseDevelopmentItems(IDevelopmentTraversable):
    """Marker interface for all development 'collection' items."""


class IDevelopmentCollection(IBaseDevelopmentItems):
    """Marker interface for DevelopmentCollection viewlet."""


class IDevelopmentDetails(Interface):
    """Marker interface for DevelopmentDetails view."""


class IDevelopmentListings(Interface):
    """Marker interface for DevelopmentListings view."""


class IPossibleDevelopmentCollection(Interface):
    """Marker interface for possible DevelopmentCollection viewlet."""


class IPossibleListingSearchBanner(Interface):
    """Marker interface for possible Listing Search Banner viewlet."""


class IListingSearchBanner(Interface):
    """Marker interface for Listing Search Banner viewlet."""


class IMLSCachingSettings(form.Schema):
    """MLS Caching Settings Schema."""

    timeout = schema.Int(
        default=3600,
        description=_(
            u'Enter the maximum time, in full seconds, that items may remain '
            u'in the cache before being requested again from the MLS. '
            u'A minimum of 60 seconds is required, a good value is 3,600 '
            u'seconds (1 hour).',
        ),
        min=60,
        required=True,
        title=_(u'Maximum age of entries in the cache'),
    )



# -*- coding: utf-8 -*-
"""Upgrade ps.plone.mls to 1010 profile."""

from plone import api


PROFILE_ID = 'profile-ps.plone.mls.upgrades:1010'


def upgrade(context, logger=None):
    setup = api.portal.get_tool(name='portal_setup')
    setup.runAllImportStepsFromProfile(PROFILE_ID)



# -*- coding: utf-8 -*-
"""Different shared utilities."""

from plone import api
from plone.api.exc import InvalidParameterError
from ps.plone.mls import _
from zope.interface import Invalid

import re


check_email = re.compile(
    r'[a-zA-Z0-9._%-]+@([a-zA-Z0-9-]+\.)*[a-zA-Z]{2,4}').match

check_for_url = re.compile(
    r'http[s]?://').search


def validate_email(value):
    if value:
        if not check_email(value):
            raise Invalid(_(u'Invalid email address.'))
    return True


def contains_nuts(value):
    """Check for traces of nuts, like urls or other spammer fun things"""
    if value:
        if check_for_url(value):
            raise Invalid(_(u'No URLs allowed.'))
    return True


def merge_local_contact_info(settings=None, mapping=None, data=None):
    """Merge values of locally provided contact info."""
    keys_internal = [
        'force',
        'use_custom_info',
    ]

    # Clear any existing data.
    data.clear()

    for key, value in settings.items():
        if key in keys_internal:
            continue
        if value is None:
            continue
        mapped_key = mapping.get(key)
        if mapped_key is None:
            continue
        data[mapped_key] = value


def smart_truncate(content):
    """Truncate a string for some max length, but split at word boundary.

    Settings for `length` and `ellipsis` are taken from the Plone settings.
    In Plone 5 the `portal_properties` tool was removed and all settings
    have been migrated to the registry. Currently the setting for
    `ellipsis` has not yet been migrated. So we have to check for both
    settings individually.
    """
    if content is None:
        return

    try:
        enabled = api.portal.get_registry_record(
            'plone.mls.listing.interfaces.IMLSUISettings.truncate_texts',
        )
    except InvalidParameterError:
        return content

    if not enabled:
        return content

    try:
        length = api.portal.get_registry_record(
            'plone.mls.listing.interfaces.IMLSUISettings.truncate_length',
        )
    except InvalidParameterError:
        length = 320

    try:
        ellipsis = api.portal.get_registry_record('plone.ellipsis')
    except InvalidParameterError:
        try:
            props = api.portal.get_tool(name='portal_properties')
            ellipsis = props.site_properties.ellipsis
        except Exception:
            ellipsis = u'...'

    if len(content) > length:
        content = content[:length].rsplit(' ', 1)[0] + ellipsis
    return content



# -*- coding: utf-8 -*-
"""Testing utilities."""

# python imports
from mls.apiclient import testing
from mls.apiclient.tests import utils

import os


def _register(endpoint, content=None, fixture=None, params=None):
    path = os.path.join(
        os.path.abspath(os.path.dirname(__file__)),
        'fixtures',
    )
    return testing._register(
        endpoint,
        content=content,
        path=path,
        fixture=fixture,
        params=params,
    )


def setup_plone_mls_fixtures():
    """Setup the test fixtures for integration tests."""
    utils.setup_listing_api_fixtures()
    utils.setup_fixtures()

    # Register Plone specific development endpoints
    _register(
        'developments',
        params=dict(
            {
                'fields': ''.join([
                    'id,title,description,logo,location,lot_size,'
                    'location_type,number_of_listings,number_of_groups',
                ]),
                'limit': 5,
                'offset': 0,
            },
            **testing.BASE_PARAMS),
        fixture='developments.json',
    )
    _register(
        'developments',
        params=dict(
            {
                'fields': ''.join([
                    'id,title,description,logo,location,lot_size,'
                    'location_type,number_of_listings,number_of_groups,'
                    'banner_image',
                ]),
                'reverse': False,
                'modify_url': True,
                'limit': 5,
                'offset': 0,
            },
            **testing.BASE_PARAMS),
        fixture='developments_banner.json',
    )



# -*- coding: utf-8 -*-
"""Featured Listings Viewlet."""

from plone import api
from plone.app.layout.viewlets.common import ViewletBase
from plone.directives import form
from plone.memoize.view import memoize
from plone.mls.core.navigation import ListingBatch
from plone.mls.listing.api import prepare_search_params
from plone.mls.listing.api import search
from plone.mls.listing.browser.interfaces import IListingDetails
from plone.supermodel import model
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ps.plone.mls import _
from ps.plone.mls import config
from ps.plone.mls import PLONE_4
from ps.plone.mls import PLONE_5
from ps.plone.mls.interfaces import IListingTraversable
from z3c.form import button
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.interface import noLongerProvides
from zope.publisher.browser import BrowserView
from zope.traversing.browser.absoluteurl import absoluteURL


CONFIGURATION_KEY = config.SETTINGS_LISTING_FEATURED


class IPossibleFeaturedListings(Interface):
    """Marker interface for possible FeaturedListings viewlet."""


class IFeaturedListings(IListingTraversable):
    """Marker interface for FeaturedListings viewlet."""


class FeaturedListingsViewlet(ViewletBase):
    """Show featured MLS listings."""

    if PLONE_5:
        index = ViewPageTemplateFile('templates/listing_results_p5.pt')
    elif PLONE_4:
        index = ViewPageTemplateFile('templates/listing_results.pt')

    @property
    def available(self):
        return IFeaturedListings.providedBy(self.context) and \
            not IListingDetails.providedBy(self.view)

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(CONFIGURATION_KEY, {})

    def update(self):
        """Prepare view related data."""
        self._listings = None
        super(FeaturedListingsViewlet, self).update()
        self.context_state = queryMultiAdapter(
            (self.context, self.request), name='plone_context_state',
        )
        self.limit = self.config.get('limit', 25)
        self._get_listings()

    def _get_listings(self):
        """Query the recent listings from the MLS."""
        listing_ids = self.config.get('listing_ids', [])
        if len(listing_ids) == 0:
            return
        listing_ids = [lid.lower() for lid in listing_ids]
        params = {
            'limit': 0,
            'offset': 0,
            'lang': self.portal_state.language(),
        }
        params.update(self.config)
        params['listing_ids'] = listing_ids
        params = prepare_search_params(params)
        results = search(params, batching=False, context=self.context)
        if results is None or len(results) == 0:
            return

        # sort the results based on the listing_ids
        results = [(item['id']['value'], item) for item in results]
        results = dict(results)
        self._listings = [
            results.get(id) for id in listing_ids if id in results
        ]

    @property
    def layout_css(self):
        if self.config.get('grid_layout', False):
            return u'listing-grid-view'
        return u''

    @property
    @memoize
    def listings(self):
        """Return listing results."""
        return self._listings

    @memoize
    def view_url(self):
        """Generate view url."""
        if not self.context_state.is_view_template():
            return self.context_state.current_base_url()
        else:
            return absoluteURL(self.context, self.request) + '/'

    @property
    def batching(self):
        return ListingBatch(
            self.listings, self.limit, self.request.get('b_start', 0),
            orphan=1,
            batch_data=None,
        )


class IFeaturedListingsConfiguration(model.Schema):
    """Featured Listings Configuration Form Schema."""

    listing_ids = schema.List(
        description=_(
            u'Add one Listing ID per line for each entry to show up.',
        ),
        title=_(u'MLS Listing IDs'),
        unique=True,
        value_type=schema.TextLine(
            title=_(u'ID'),
        ),
    )

    grid_layout = schema.Bool(
        description=_(
            u'If allowed by the theme/design, the listings will be displayed '
            u'in a grid layout when enabled.',
        ),
        default=False,
        required=False,
        title=_(u'Grid Layout'),
    )


class FeaturedListingsConfiguration(form.SchemaForm):
    """Featured Listings Configuration Form."""

    schema = IFeaturedListingsConfiguration

    label = _(u'\'Featured Listings\' Configuration')
    description = _(
        u'Adjust the behaviour for this \'Featured Listings\' viewlet.',
    )

    def getContent(self):
        annotations = IAnnotations(self.context)
        return annotations.get(
            CONFIGURATION_KEY, annotations.setdefault(CONFIGURATION_KEY, {}),
        )

    @button.buttonAndHandler(_(u'Save'))
    def handle_save(self, action):
        data, errors = self.extractData()
        if not errors:
            annotations = IAnnotations(self.context)
            annotations[CONFIGURATION_KEY] = data
            self.request.response.redirect(
                absoluteURL(self.context, self.request),
            )

    @button.buttonAndHandler(_(u'Cancel'))
    def handle_cancel(self, action):
        self.request.response.redirect(absoluteURL(self.context, self.request))


class FeaturedListingsStatus(object):
    """Return activation/deactivation status of FeaturedListings viewlet."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def can_activate(self):
        return IPossibleFeaturedListings.providedBy(self.context) and \
            not IFeaturedListings.providedBy(self.context)

    @property
    def active(self):
        return IFeaturedListings.providedBy(self.context)


class FeaturedListingsToggle(object):
    """Toggle FeaturedListings viewlet for the current context."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        msg_type = 'info'

        if IFeaturedListings.providedBy(self.context):
            # Deactivate FeaturedListings viewlet.
            noLongerProvides(self.context, IFeaturedListings)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u'\'Featured Listings\' viewlet deactivated.')
        elif IPossibleFeaturedListings.providedBy(self.context):
            alsoProvides(self.context, IFeaturedListings)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u'\'Featured Listings\' viewlet activated.')
        else:
            msg = _(
                u'The \'Featured Listings\' viewlet does\'t work with this '
                u'content type. Add \'IPossibleFeaturedListings\' to the '
                u'provided interfaces to enable this feature.',
            )
            msg_type = 'error'

        api.portal.show_message(
            message=msg, request=self.request, type=msg_type,
        )
        self.request.response.redirect(self.context.absolute_url())


class FeaturedListings(BrowserView):
    """Featured Listings view"""

    limit = 25

    def __init__(self, context, request):
        super(FeaturedListings, self).__init__(context, request)
        self.portal_state = queryMultiAdapter(
            (self.context, self.request), name='plone_portal_state',
        )

    def _get_listings(self):
        """Query the recent listings from the MLS."""
        listing_ids = self.context.listing_ids
        if len(listing_ids) == 0:
            return
        listing_ids = [lid.lower() for lid in listing_ids]
        params = {
            'limit': 0,
            'offset': 0,
            'lang': self.portal_state.language(),
        }
        params.update({
            'listing_ids': listing_ids,
        })
        params = prepare_search_params(params)
        results = search(params, batching=False, context=self.context)
        if results is None or len(results) == 0:
            return

        # sort the results based on the listing_ids
        results = [(item['id']['value'], item) for item in results]
        results = dict(results)
        return [results.get(id) for id in listing_ids if id in results]

    @property
    @memoize
    def listings(self):
        """Return listing results."""
        return self._get_listings()

    @property
    def batching(self):
        return ListingBatch(
            self.listings,
            self.limit,
            self.request.get('b_start', 0),
            orphan=1,
            batch_data=None,
        )



# -*- coding: utf-8 -*-
"""MLS Plone Embedding Control Panel."""

from plone.app.registry.browser import controlpanel
from plone.registry.interfaces import IRegistry
from plone.z3cform import layout
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ps.plone.mls import _
from ps.plone.mls import interfaces
from z3c.form import field
from zope.component import getUtility
from zope.interface import implementer


class SelfHealingRegistryEditForm(controlpanel.RegistryEditForm):
    """Registers the schema if an error occured."""

    def getContent(self):
        registry = getUtility(IRegistry)
        try:
            return registry.forInterface(  # noqa
                self.schema,
                prefix=self.schema_prefix,
            )
        except KeyError:
            self.ignoreContext = True
            self.fields = field.Fields()
            registry.registerInterface(self.schema)
            self.status = _(
                u'Registry has been updated. Please reload this page.',
            )
            return None


class MLSBaseSettingsControlPanel(SelfHealingRegistryEditForm):
    schema = interfaces.IMLSBaseSettings
    label = _(u'Propertyshelf MLS Base Settings')
    description = _(
        u'This MLS configuration will be used as the default for this '
        u'Plone site. You can add more MLS configurations by activating '
        u'the local MLS settings on any content item within the site.',
    )


class MLSBaseSettingsFormWrapper(layout.FormWrapper):
    """Use this form as the layout wrapper to get the controlpanel layout."""

    index = ViewPageTemplateFile('controlpanel_layout_base.pt')


MLSBaseSettingsPanelView = layout.wrap_form(
    MLSBaseSettingsControlPanel,
    MLSBaseSettingsFormWrapper,
)


class MLSUISettingsControlPanel(SelfHealingRegistryEditForm):
    schema = interfaces.IMLSUISettings
    label = _(u'Propertyshelf MLS UI Settings')
    description = _(
        u'Adjust the UI settings for this Plone site.',
    )


class MLSUISettingsFormWrapper(layout.FormWrapper):
    """Use this form as the layout wrapper to get the controlpanel layout."""

    index = ViewPageTemplateFile('controlpanel_layout_ui.pt')


MLSUISettingsPanelView = layout.wrap_form(
    MLSUISettingsControlPanel,
    MLSUISettingsFormWrapper,
)


@implementer(interfaces.IGlobalContactSettingsForm)
class MLSContactInfoSettingsControlPanel(SelfHealingRegistryEditForm):
    schema = interfaces.IMLSContactInfoSettings
    label = _(u'Propertyshelf MLS Contact Information Settings')
    description = _(
        u'The contact information provided will be used as the default for '
        u'this Plone site. You can add more specific MLS contact '
        u'information by activating the local MLS contact settings on any '
        u'content item within the site.',
    )


class MLSContactInfoSettingsFormWrapper(layout.FormWrapper):
    """Use this form as the layout wrapper to get the controlpanel layout."""

    index = ViewPageTemplateFile('controlpanel_layout_contact_info.pt')


MLSContactInfoSettingsPanelView = layout.wrap_form(
    MLSContactInfoSettingsControlPanel,
    MLSContactInfoSettingsFormWrapper,
)


class MLSCachingSettingsControlPanel(SelfHealingRegistryEditForm):
    schema = interfaces.IMLSCachingSettings
    label = _(u'Propertyshelf MLS Caching Settings')
    description = _(u'Adjust the MLS caching behhavior for this Plone site.')


class MLSCachingSettingsFormWrapper(layout.FormWrapper):
    """Use this form as the layout wrapper to get the controlpanel layout."""

    index = ViewPageTemplateFile('controlpanel_layout_caching.pt')


MLSCachingSettingsPanelView = layout.wrap_form(
    MLSCachingSettingsControlPanel,
    MLSCachingSettingsFormWrapper,
)



# -*- coding: utf-8 -*-
"""Listing search banner."""

from Acquisition import aq_inner
from plone import api
from plone.app.layout.viewlets.common import ViewletBase
from plone.directives import form
from plone.formwidget.namedfile.converter import b64decode_file
from plone.formwidget.namedfile.widget import NamedImageFieldWidget
from plone.namedfile.browser import Download
from plone.namedfile.file import NamedImage
from plone.supermodel.directives import fieldset
from plone.z3cform import z2
from Products.CMFPlone import PloneMessageFactory as PMF
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ps.plone.mls import _
from ps.plone.mls import config
from ps.plone.mls import PLONE_4
from ps.plone.mls import PLONE_5
from ps.plone.mls.interfaces import IListingSearchBanner
from ps.plone.mls.interfaces import IPossibleListingSearchBanner
from z3c.form import button
from z3c.form import field
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import StaticWidgetAttribute
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.interface import alsoProvides
from zope.interface import noLongerProvides
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import copy
import urlparse


try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

# starting from 0.6.0 version plone.z3cform has IWrappedForm interface
try:
    from plone.z3cform.interfaces import IWrappedForm
    HAS_WRAPPED_FORM = True
except ImportError:
    HAS_WRAPPED_FORM = False


FIELDS_SECTION_1 = [
    'section_1_search_target',
    'section_1_title',
    'section_1_categories',
    'section_1_default_category',
    'section_1_hide_categories',
    'section_1_hide_beds',
    'section_1_hide_baths',
    'section_1_hide_price',
    'section_1_hide_section',
]
FIELDS_SECTION_2 = [
    'section_2_search_target',
    'section_2_title',
    'section_2_categories',
    'section_2_default_category',
    'section_2_hide_categories',
    'section_2_hide_beds',
    'section_2_hide_baths',
    'section_2_hide_price',
    'section_2_hide_section',
]
FIELDS_SECTION_3 = [
    'section_3_search_target',
    'section_3_title',
    'section_3_categories',
    'section_3_default_category',
    'section_3_hide_categories',
    'section_3_hide_beds',
    'section_3_hide_baths',
    'section_3_hide_price',
    'section_3_hide_section',
]
FIELDS_SECTION_4 = [
    'section_4_search_target',
    'section_4_title',
    'section_4_categories',
    'section_4_default_category',
    'section_4_hide_categories',
    'section_4_hide_beds',
    'section_4_hide_baths',
    'section_4_hide_price',
    'section_4_hide_section',
]
FIELDS_UI = [
    'image',
    'image_url',
    'image_height',
]

LABEL_SECTION_SEARCH_PAGE = _(u'Search Target Page')
LABEL_SECTION_TITLE = _(u'Title')
LABEL_SECTION_CATEGORIES = _(u'Categories')
LABEL_SECTION_DEFAULT_CATEGORY = _(u'Default Category')
LABEL_SECTION_HIDE_CATEGORY = _(u'Hide category')
LABEL_SECTION_HIDE_BEDS = _(u'Hide bedrooms')
LABEL_SECTION_HIDE_BATHS = _(u'Hide bathrooms')
LABEL_SECTION_HIDE_PRICE = _(u'Hide price')
LABEL_SECTION_HIDE_SECTION = _(u'Hide section')

DESCRIPTION_SECTION_SEARCH_PAGE = _(
    u'Select the activated listing search page which will be used to show '
    u'the results.',
)
DESCRIPTION_SECTION_TITLE = _(u'')
DESCRIPTION_SECTION_CATEGORIES = _(u'')
DESCRIPTION_SECTION_DEFAULT_CATEGORY = _(u'')
DESCRIPTION_SECTION_HIDE_CATEGORY = _(
    u'Hide the category field and use default setting.',
)
DESCRIPTION_SECTION_HIDE_BEDS = _(
    u'Hide the bedrooms field.',
)
DESCRIPTION_SECTION_HIDE_BATHS = _(
    u'Hide the bathrooms field.',
)
DESCRIPTION_SECTION_HIDE_PRICE = _(
    u'Hide the price min and price max field.',
)
DESCRIPTION_SECTION_HIDE_SECTION = _(
    u'Don\'t show this section at all.',
)

DEFAULT_CATEGORIES_1 = (
    u'all:All:listing_type=rs,cs,ll\n'
    u'houses:Homes:listing_type=rs&object_type=house,mobile,multiplex,'
    u'townhouse,freestanding_villa\n'
    u'condos:Condos:listing_type=rs&object_type=apartment,condominium,'
    u'half_duplex\n'
    u'land:Land:listing_type=ll\n'
    u'commercial:Commercial:listing_type=cs&object_type=\n'
)

DEFAULT_CATEGORIES_2 = (
    u'all:All:listing_type=rl,cl\n'
    u'houses:Homes:listing_type=rl&object_type=house,mobile,multiplex,'
    u'townhouse,freestanding_villa\n'
    u'condos:Condos:listing_type=rl&object_type=apartment,condominium,'
    u'half_duplex\n'
    u'commercial:Commercial:listing_type=cl&object_type=\n'
)


class ISectionForm(form.Schema):
    """Section Search Form schema."""

    q = schema.TextLine(
        required=False,
        title=_(u'Location, Keywords, Listing ID, ...'),
    )

    category = schema.Choice(
        required=True,
        title=_(u'Category'),
        values=['one', 'two'],
    )

    beds = schema.Choice(
        required=False,
        title=_(u'Bedrooms'),
        source='ps.plone.mls.listings.min_bedrooms',
    )

    baths = schema.Choice(
        required=False,
        title=_(u'Baths'),
        source='ps.plone.mls.listings.min_bedrooms',
    )

    price_min = schema.Int(
        required=False,
        title=_(u'Price (Min)'),
    )

    price_max = schema.Int(
        required=False,
        title=_(u'Price (Max)'),
    )


class SectionForm(form.Form):
    """Section Search Form."""

    fields = field.Fields(ISectionForm)
    ignoreContext = True
    method = 'post'

    def __init__(self, context, request, config=None, prefix=None):
        super(SectionForm, self).__init__(context, request)
        self.config = config
        self.categories = None
        self.category_queries = {}
        if prefix is not None and isinstance(prefix, basestring):
            self.prefix = 'form.{0}.'.format(prefix)

    @button.buttonAndHandler(PMF(u'label_search', default=u'Search'),
                             name='search')
    def handle_search(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        url = '?'.join([
            self.search_target_url,
            self.prepare_query_string(data),
        ])
        self.request.response.redirect(url)

    def _prepare_category_search(self, query):
        result = []
        prefix = 'form.{0}.widgets.'.format(self.search_target_id)
        params = dict(urlparse.parse_qsl(query))
        for key, value in params.items():
            values = value.split(',')
            if len(values) > 1:
                for item in values:
                    result.append(urlencode({
                        prefix + key + ':list': item,
                    }))
            else:
                result.append(urlencode({prefix + key: value}))
        return '&'.join(result)

    def prepare_query_string(self, data=None):
        prefix = 'form.{0}.widgets.'.format(self.search_target_id)
        category_query = ''
        query = {
            'form.{0}.buttons.search'.format(self.search_target_id): '',
        }
        if not data:
            return ''
        beds = data.pop('beds', None)
        if beds:
            query[prefix + 'beds-min'] = beds
            query[prefix + 'beds-max'] = '--MAXVALUE--'
        baths = data.pop('baths', None)
        if baths:
            query[prefix + 'baths-min'] = baths
            query[prefix + 'baths-max'] = '--MAXVALUE--'
        category = data.pop('category', None)
        if self.config.get('hide_categories', False):
            category = self.config.get('default_category', None)
        if category:
            q = self.category_queries.get(category)
            category_query = self._prepare_category_search(q)
        for key, value in data.items():
            if value is not None:
                query[prefix + key] = value
        return '&'.join([
            urlencode(query),
            category_query,
        ])

    @property
    def search_target_url(self):
        target_uid = self.config.get('search_target', None)
        if not target_uid:
            return
        obj = api.content.get(UID=target_uid)
        return obj.absolute_url()

    @property
    def search_target_id(self):
        target_uid = self.config.get('search_target', None)
        if not target_uid:
            return
        obj = api.content.get(UID=target_uid)
        return obj.id

    def update(self):
        """Update form to match configuration."""
        omitted = []
        self._generate_categories()
        if self.config.get('hide_categories', False) or not self.categories:
            omitted.append('category')
        if self.config.get('hide_beds', False):
            omitted.append('beds')
        if self.config.get('hide_baths', False):
            omitted.append('baths')
        if self.config.get('hide_price', False):
            omitted.append('price_min')
            omitted.append('price_max')
        self.fields = field.Fields(ISectionForm).omit(*omitted)
        self.update_fields()
        super(SectionForm, self).update()
        if PLONE_5:
            if 'category' in self.widgets:
                self.widgets['category'].pattern_options = {
                    'placeholder': _(u'Select a Category'),
                }
            if 'beds' in self.widgets:
                self.widgets['beds'].pattern_options = {
                    'placeholder': _(u'Beds'),
                }
            if 'baths' in self.widgets:
                self.widgets['baths'].pattern_options = {
                    'placeholder': _(u'Baths'),
                }

    def update_fields(self):
        """Update form field configurations."""
        if 'category' in self.fields:
            field = self.fields['category']
            field.field.vocabulary = self.categories
            default = self.config.get('default_category', None)
            if default not in self.category_queries:
                default = list(self.categories)[0].token
            field.field.default = default

        if PLONE_5:
            from plone.app.z3cform.widget import SelectFieldWidget

            if 'category' in self.fields:
                self.fields['category'].widgetFactory = SelectFieldWidget
            if 'beds' in self.fields:
                self.fields['beds'].widgetFactory = SelectFieldWidget
            if 'baths' in self.fields:
                self.fields['baths'].widgetFactory = SelectFieldWidget

    def _generate_categories(self):
        """Return a new categories vocabulary."""
        categories = self.config.get('categories', None)
        if not categories:
            return
        terms = []
        for category in categories.splitlines():
            try:
                key, title, query = category.split(':')
            except ValueError:
                continue
            else:
                terms.append(SimpleTerm(key, key, title))
                self.category_queries[key] = query

        self.categories = SimpleVocabulary(terms)


class SearchBanner(ViewletBase):
    """Listing Search Banner."""

    if PLONE_5:
        index = ViewPageTemplateFile('templates/search_banner_p5.pt')
    elif PLONE_4:
        index = ViewPageTemplateFile('templates/search_banner.pt')
    config = None
    section_1 = None
    section_2 = None
    section_3 = None
    section_4 = None

    @property
    def available(self):
        """Check if this viewlet is available for rendering."""
        return IListingSearchBanner.providedBy(self.context) and (
            self.section_1 or self.section_2 or
            self.section_3 or self.section_4
        )

    def get_config(self):
        """Get the configuration data from annotations."""
        annotations = IAnnotations(self.context)
        self.config = copy.deepcopy(
            annotations.get(config.SETTINGS_LISTING_SEARCH_BANNER, {}),
        )

    @property
    def image_url(self):
        """Return the image URL for the background image."""
        image = self.config.get('image', None)
        if image is not None:
            filename, data = b64decode_file(image)
            return '{0}/@@listing-search-banner-image/{1}'.format(
                self.context.absolute_url(),
                filename,
            )
        image_url = self.config.get('image_url', None)
        if image_url is not None and image_url != u'http://':
            return image_url

    def map_config(self, section=None):
        """Return a config specific for a section."""
        config = {}
        if section is None:
            return
        if not isinstance(section, basestring):
            return
        for key in self.config.keys():
            if key.startswith(section):
                new_key = key[len(section) + 1:]
                config[new_key] = self.config.get(key)
        return config

    def setup_forms(self):
        """Initialize the section forms."""
        z2.switch_on(self, request_layer=IFormLayer)
        if not self.config.get('section_1_hide_section', False) and \
                self.config.get('section_1_search_target', None):
            self.section_1 = SectionForm(
                aq_inner(self.context),
                self.request,
                config=self.map_config('section_1'),
                prefix='section_1',
            )
            if HAS_WRAPPED_FORM:
                alsoProvides(self.section_1, IWrappedForm)
            self.section_1.update()
        if not self.config.get('section_2_hide_section', False) and \
                self.config.get('section_2_search_target', None):
            self.section_2 = SectionForm(
                aq_inner(self.context),
                self.request,
                config=self.map_config('section_2'),
                prefix='section_2',
            )
            if HAS_WRAPPED_FORM:
                alsoProvides(self.section_2, IWrappedForm)
            self.section_2.update()
        if not self.config.get('section_3_hide_section', False) and \
                self.config.get('section_3_search_target', None):
            self.section_3 = SectionForm(
                aq_inner(self.context),
                self.request,
                config=self.map_config('section_3'),
                prefix='section_3',
            )
            if HAS_WRAPPED_FORM:
                alsoProvides(self.section_3, IWrappedForm)
            self.section_3.update()
        if not self.config.get('section_4_hide_section', False) and \
                self.config.get('section_4_search_target', None):
            self.section_4 = SectionForm(
                aq_inner(self.context),
                self.request,
                config=self.map_config('section_4'),
                prefix='section_4',
            )
            if HAS_WRAPPED_FORM:
                alsoProvides(self.section_4, IWrappedForm)
            self.section_4.update()

    def update_config(self):
        if not self.config.get('section_1_title', None):
            self.config['section_1_title'] = u'Section 1'
        if not self.config.get('section_2_title', None):
            self.config['section_2_title'] = u'Section 2'
        if not self.config.get('section_3_title', None):
            self.config['section_3_title'] = u'Section 3'
        if not self.config.get('section_4_title', None):
            self.config['section_4_title'] = u'Section 4'

    def update(self):
        """Prepare view related data."""
        super(SearchBanner, self).update()
        self.get_config()
        self.update_config()
        self.setup_forms()


class ISearchBannerConfiguration(form.Schema):
    """Listing Search Banner configuration form schema."""

    fieldset('section_1', label=_(u'Section 1'), fields=FIELDS_SECTION_1)
    fieldset('section_2', label=_(u'Section 2'), fields=FIELDS_SECTION_2)
    fieldset('section_3', label=_(u'Section 3'), fields=FIELDS_SECTION_3)
    fieldset('section_4', label=_(u'Section 4'), fields=FIELDS_SECTION_4)
    fieldset('ui', label=_(u'UI'), fields=FIELDS_UI)

    form.widget(image=NamedImageFieldWidget)
    image = schema.ASCII(
        description=_(
            u'Upload an image file which will be used as a background for '
            u'your listing banner search.',
        ),
        required=False,
        title=_(u'Banner Image'),
    )

    image_url = schema.TextLine(
        default=u'http://',
        description=_(
            u'Instead of uploading an image, you may enter the URL of an '
            u'image hosted on another server.',
        ),
        max_length=511,
        required=False,
        title=_(u'Banner Image URL'),
    )

    image_height = schema.Int(
        default=450,
        required=False,
        description=_(
            u'Enter the image height in pixels. The default is 450 and the '
            u'recommended amount is between 350 and 600 pixels.',
        ),
        min=350,
        max=1000,
        title=_(u'Banner Image Height'),
    )

    section_1_search_target = schema.Choice(
        description=DESCRIPTION_SECTION_SEARCH_PAGE,
        required=False,
        vocabulary='ps.plone.mls.listings.available_searches',
        title=LABEL_SECTION_SEARCH_PAGE,
    )

    section_1_title = schema.TextLine(
        default=_(u'Buy'),
        description=DESCRIPTION_SECTION_TITLE,
        required=False,
        title=LABEL_SECTION_TITLE,
    )

    try:
        form.widget('section_1_categories', rows=6)
    except TypeError:
        pass
    section_1_categories = schema.Text(
        default=DEFAULT_CATEGORIES_1,
        description=DESCRIPTION_SECTION_CATEGORIES,
        required=False,
        title=LABEL_SECTION_CATEGORIES,
    )

    section_1_default_category = schema.TextLine(
        default=u'houses',
        description=DESCRIPTION_SECTION_DEFAULT_CATEGORY,
        required=False,
        title=LABEL_SECTION_DEFAULT_CATEGORY,
    )

    section_1_hide_categories = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_CATEGORY,
        required=False,
        title=LABEL_SECTION_HIDE_CATEGORY,
    )

    section_1_hide_beds = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BEDS,
        required=False,
        title=LABEL_SECTION_HIDE_BEDS,
    )

    section_1_hide_baths = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BATHS,
        required=False,
        title=LABEL_SECTION_HIDE_BATHS,
    )

    section_1_hide_price = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_PRICE,
        required=False,
        title=LABEL_SECTION_HIDE_PRICE,
    )

    section_1_hide_section = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_SECTION,
        required=False,
        title=LABEL_SECTION_HIDE_SECTION,
    )

    section_2_search_target = schema.Choice(
        description=DESCRIPTION_SECTION_SEARCH_PAGE,
        required=False,
        vocabulary='ps.plone.mls.listings.available_searches',
        title=LABEL_SECTION_SEARCH_PAGE,
    )

    section_2_title = schema.TextLine(
        default=_(u'Rent'),
        description=DESCRIPTION_SECTION_TITLE,
        required=False,
        title=LABEL_SECTION_TITLE,
    )

    try:
        form.widget('section_2_categories', rows=6)
    except TypeError:
        pass
    section_2_categories = schema.Text(
        default=DEFAULT_CATEGORIES_2,
        description=DESCRIPTION_SECTION_CATEGORIES,
        required=False,
        title=LABEL_SECTION_CATEGORIES,
    )

    section_2_default_category = schema.TextLine(
        default=u'houses',
        description=DESCRIPTION_SECTION_DEFAULT_CATEGORY,
        required=False,
        title=LABEL_SECTION_DEFAULT_CATEGORY,
    )

    section_2_hide_categories = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_CATEGORY,
        required=False,
        title=LABEL_SECTION_HIDE_CATEGORY,
    )

    section_2_hide_beds = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BEDS,
        required=False,
        title=LABEL_SECTION_HIDE_BEDS,
    )

    section_2_hide_baths = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BATHS,
        required=False,
        title=LABEL_SECTION_HIDE_BATHS,
    )

    section_2_hide_price = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_PRICE,
        required=False,
        title=LABEL_SECTION_HIDE_PRICE,
    )

    section_2_hide_section = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_SECTION,
        required=False,
        title=LABEL_SECTION_HIDE_SECTION,
    )

    section_3_search_target = schema.Choice(
        description=DESCRIPTION_SECTION_SEARCH_PAGE,
        required=False,
        vocabulary='ps.plone.mls.listings.available_searches',
        title=LABEL_SECTION_SEARCH_PAGE,
    )

    section_3_title = schema.TextLine(
        description=DESCRIPTION_SECTION_TITLE,
        required=False,
        title=LABEL_SECTION_TITLE,
    )

    try:
        form.widget('section_3_categories', rows=6)
    except TypeError:
        pass
    section_3_categories = schema.Text(
        description=DESCRIPTION_SECTION_CATEGORIES,
        required=False,
        title=LABEL_SECTION_CATEGORIES,
    )

    section_3_default_category = schema.TextLine(
        description=DESCRIPTION_SECTION_DEFAULT_CATEGORY,
        required=False,
        title=LABEL_SECTION_DEFAULT_CATEGORY,
    )

    section_3_hide_categories = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_CATEGORY,
        required=False,
        title=LABEL_SECTION_HIDE_CATEGORY,
    )

    section_3_hide_beds = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BEDS,
        required=False,
        title=LABEL_SECTION_HIDE_BEDS,
    )

    section_3_hide_baths = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BATHS,
        required=False,
        title=LABEL_SECTION_HIDE_BATHS,
    )

    section_3_hide_price = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_PRICE,
        required=False,
        title=LABEL_SECTION_HIDE_PRICE,
    )

    section_3_hide_section = schema.Bool(
        default=True,
        description=DESCRIPTION_SECTION_HIDE_SECTION,
        required=False,
        title=LABEL_SECTION_HIDE_SECTION,
    )

    section_4_search_target = schema.Choice(
        description=DESCRIPTION_SECTION_SEARCH_PAGE,
        required=False,
        vocabulary='ps.plone.mls.listings.available_searches',
        title=LABEL_SECTION_SEARCH_PAGE,
    )

    section_4_title = schema.TextLine(
        description=DESCRIPTION_SECTION_TITLE,
        required=False,
        title=LABEL_SECTION_TITLE,
    )

    try:
        form.widget('section_4_categories', rows=6)
    except TypeError:
        pass
    section_4_categories = schema.Text(
        description=DESCRIPTION_SECTION_CATEGORIES,
        required=False,
        title=LABEL_SECTION_CATEGORIES,
    )

    section_4_default_category = schema.TextLine(
        description=DESCRIPTION_SECTION_DEFAULT_CATEGORY,
        required=False,
        title=LABEL_SECTION_DEFAULT_CATEGORY,
    )

    section_4_hide_categories = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_CATEGORY,
        required=False,
        title=LABEL_SECTION_HIDE_CATEGORY,
    )

    section_4_hide_beds = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BEDS,
        required=False,
        title=LABEL_SECTION_HIDE_BEDS,
    )

    section_4_hide_baths = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_BATHS,
        required=False,
        title=LABEL_SECTION_HIDE_BATHS,
    )

    section_4_hide_price = schema.Bool(
        description=DESCRIPTION_SECTION_HIDE_PRICE,
        required=False,
        title=LABEL_SECTION_HIDE_PRICE,
    )

    section_4_hide_section = schema.Bool(
        default=True,
        description=DESCRIPTION_SECTION_HIDE_SECTION,
        required=False,
        title=LABEL_SECTION_HIDE_SECTION,
    )


class SearchBannerConfiguration(form.SchemaForm):
    """Listing Search Banner Configuration Form."""

    schema = ISearchBannerConfiguration
    label = _(u'\'Listing Search Banner\' Configuration')
    description = _(
        u'Adjust the behaviour for this \'Listing Search Banner\'.',
    )

    def getContent(self):
        annotations = IAnnotations(self.context)
        return annotations.get(
            config.SETTINGS_LISTING_SEARCH_BANNER,
            annotations.setdefault(config.SETTINGS_LISTING_SEARCH_BANNER, {}),
        )

    @button.buttonAndHandler(_(u'Save'))
    def handle_save(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        annotations = IAnnotations(self.context)
        annotations[config.SETTINGS_LISTING_SEARCH_BANNER] = data
        api.portal.show_message(
            message=PMF(u'Changes saved.'),
            request=self.request,
        )


class SearchBannerStatus(object):
    """Return activation/deactivation status of the viewlet."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def can_activate(self):
        return IPossibleListingSearchBanner.providedBy(self.context) and \
            not IListingSearchBanner.providedBy(self.context)

    @property
    def active(self):
        return IListingSearchBanner.providedBy(self.context)


class SearchBannerToggle(object):
    """Toggle listing search banner viewlet for the current context."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        msg_type = 'info'

        if IListingSearchBanner.providedBy(self.context):
            # Deactivate search banner viewlet.
            noLongerProvides(self.context, IListingSearchBanner)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u'\'Listing Search Banner\' deactivated.')
        elif IPossibleListingSearchBanner.providedBy(self.context):
            alsoProvides(self.context, IListingSearchBanner)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u'\'Listing Search Banner\' activated.')
        else:
            msg = _(
                u'The \'Listing Search Banner\' does\'t work with '
                u'this content type. Add \'IPossibleListingSearchBanner\' '
                u'to the provided interfaces to enable this feature.',
            )
            msg_type = 'error'

        api.portal.show_message(
            request=self.request,
            message=msg,
            type=msg_type,
        )
        self.request.response.redirect(self.context.absolute_url())


class BannerImage(Download):
    """Banner image download view."""

    def __init__(self, context, request):
        super(BannerImage, self).__init__(context, request)
        self.filename = None
        self.data = None

        image = self.config.get('image', None)
        if image is not None:
            filename, data = b64decode_file(image)
            data = NamedImage(data=data, filename=filename)
            self.data = data
            self.filename = filename
            # self.width, self.height = self.data.getImageSize()

    @property
    def config(self):
        """Get the configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_LISTING_SEARCH_BANNER, {})

    def _getFile(self):
        return self.data


NoValueBedrooms = StaticWidgetAttribute(
    _('Beds'),
    field=ISectionForm['beds'],
)

NoValueBathrooms = StaticWidgetAttribute(
    _('Baths'),
    field=ISectionForm['baths'],
)



# -*- coding: utf-8 -*-
"""Configuration options for the Propertyshelf MLS Plone Embedding."""

from plone.mls.core.browser import localconfig as localconfig_core
from plone.mls.listing.browser import listing_collection
from plone.mls.listing.browser import listing_search
from plone.mls.listing.browser import localconfig
from plone.mls.listing.browser import recent_listings


PROFILE_ID = u'profile-ps.plone.mls'
INSTALL_PROFILE = '{0}:default'.format(PROFILE_ID)
UNINSTALL_PROFILE = '{0}:uninstall'.format(PROFILE_ID)
PROJECT_NAME = 'ps.plone.mls'

#: Configuration key for development collection settings.
SETTINGS_DEVELOPMENT_COLLECTION = 'ps.plone.mls.developmentcollection'

#: Configuration key for featured listings settings.
SETTINGS_LISTING_FEATURED = 'ps.plone.mls.listing.featuredlistings'

#: Configuration key for listing search banner settings.
SETTINGS_LISTING_SEARCH_BANNER = 'ps.plone.mls.listingsearchbanner'

SETTINGS_LOCAL_AGENCY = localconfig.CONFIGURATION_KEY
SETTINGS_LOCAL_MLS = localconfig_core.CONFIGURATION_KEY
SETTINGS_LISTING_COLLECTION = listing_collection.CONFIGURATION_KEY
SETTINGS_LISTING_RECENT = recent_listings.CONFIGURATION_KEY
SETTINGS_LISTING_SEARCH = listing_search.CONFIGURATION_KEY



# -*- coding: utf-8 -*-
"""MLS development detail view."""

from Acquisition import aq_inner
from email import message_from_string
from email.utils import formataddr
from email.utils import getaddresses
from mls.apiclient import exceptions
from plone import api as plone_api
from plone.app.layout.viewlets.common import ViewletBase
from plone.directives import form
from plone.formwidget.captcha.validator import CaptchaValidator
from plone.formwidget.captcha.validator import WrongCaptchaCode
from plone.formwidget.captcha.widget import CaptchaFieldWidget
from plone.memoize.view import memoize
from plone.mls.core.navigation import ListingBatch
from plone.registry.interfaces import IRegistry  # noqa
from plone.z3cform import z2
from Products.CMFPlone import PloneMessageFactory as PMF
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ps.plone.mls import _
from ps.plone.mls import api
from ps.plone.mls import config
from ps.plone.mls import PLONE_4
from ps.plone.mls import PLONE_5
from ps.plone.mls import utils
from ps.plone.mls.interfaces import IDevelopmentDetails
from z3c.form import button
from z3c.form import field
from z3c.form import validator
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import IFormLayer
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import implementer

import json
import logging
import pkg_resources


try:
    from plone.mls.listing.interfaces import IMLSUISettings
    HAS_UI_SETTINGS = True
except ImportError:
    HAS_UI_SETTINGS = False

# starting from 0.6.0 version plone.z3cform has IWrappedForm interface
try:
    from plone.z3cform.interfaces import IWrappedForm
    HAS_WRAPPED_FORM = True
except ImportError:
    HAS_WRAPPED_FORM = False


logger = logging.getLogger(config.PROJECT_NAME)

EMAIL_TEMPLATE = _(
    u'development_contact_email',
    default=u'Enquiry from: {name} <{sender_from_address}>\n'
    u'Development URL: {url}\n'
    u'\n'
    u'Phone Number: {phone}\n'
    u'\n'
    u'Message:\n'
    u'{message}\n',
)

EMAIL_TEMPLATE_AGENT = _(
    u'development_contact_email_agent',
    default=u'The responsible agent for this development is '
    u'{agent_name}.\n'
    u'\n'
    u'Please contact {agent_name} at {agent_email}',
)


MAP_JS = """
var isTouch = false;
var map;

window.addEventListener('touchmove', function MoveDetector(){{
    isTouch = true;
    window.removeEventListener('touchmove', MoveDetector);
    map = initializeMap();
}});

function loadScript(src, callback) {{
  var script = document.createElement("script");
  script.type = "text/javascript";
  if (callback) {{
    script.onload = callback;
  }}
  document.getElementsByTagName("head")[0].appendChild(script);
  script.src = src;
}}


function loadGoogleMaps(callback) {{
  if (typeof google === 'object' && typeof google.maps === 'object') {{
    callback();
  }} else {{
    loadScript('https://maps.googleapis.com/maps/api/js?key={ak}', callback);
  }}
}}


function initializeMap() {{
    var center = new google.maps.LatLng({lat}, {lng})

    var myOptions = {{
        zoom: {zoom},
        center: center,
        mapTypeId: google.maps.MapTypeId.TERRAIN,
        mapTypeControl: true,
        disableDoubleClickZoom: true,
        overviewMapControl: true,
        streetViewControl: true,
        scrollwheel: false,
        draggable:!isTouch
    }}

    var map = new google.maps.Map(
        document.getElementById('{map_id}'),
        myOptions
    );

    var has_marker = true;
    if(has_marker) {{
        var myLatlng = new google.maps.LatLng({lat}, {lng});
        var marker = new google.maps.Marker({{
            position: myLatlng,
            map: map,
            icon: {icon}
        }});
    }}

    google.maps.event.addDomListener(window, "resize", function() {{
        var center = map.getCenter();
        google.maps.event.trigger(map, "resize");
        map.setCenter(center);
    }});

    return map;
}};

loadGoogleMaps(initializeMap);

"""


class IContactForm(form.Schema):
    """Contact Form schema."""

    sender_from_address = schema.TextLine(
        constraint=utils.validate_email,
        description=PMF(
            u'help_sender_from_address',
            default=u'',
        ),
        required=True,
        title=PMF(u'label_sender_from_address', default=u'E-Mail'),
    )

    name = schema.TextLine(
        description=PMF(
            u'help_sender_fullname',
            default=u'',
        ),
        required=True,
        title=PMF(u'label_name', default=u'Name'),
    )

    phone = schema.TextLine(
        required=False,
        title=PMF(u'label_phone', default=u'Phone'),
    )

    message = schema.Text(
        constraint=utils.contains_nuts,
        description=PMF(
            u'help_message',
            default=u'',
        ),
        max_length=1000,
        required=True,
        title=PMF(u'label_message', default=u'Message'),
    )

    form.widget(captcha=CaptchaFieldWidget)
    captcha = schema.TextLine(
        required=True,
        title=_(u'Captcha'),
    )

    subject = schema.TextLine(
        required=False,
        title=PMF(u'label_subject', default=u'Subject'),
    )


class ContactForm(form.Form):
    """Contact Form."""

    fields = field.Fields(IContactForm)
    ignoreContext = True
    method = 'post'
    _email_sent = False
    fields['captcha'].widgetFactory = CaptchaFieldWidget
    email_override = None

    def __init__(self, context, request, info=None, development=None):
        super(ContactForm, self).__init__(context, request)
        self.item_info = info
        self.development = development

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_DEVELOPMENT_COLLECTION, {})

    def update(self):
        if self.config.get('show_captcha', False) is False:
            self.fields = field.Fields(IContactForm).omit('captcha')

        email_override = self.config.get('contact_override', None)

        if email_override is not None:
            self.email_override = email_override

        super(ContactForm, self).update()

    def updateWidgets(self):
        super(ContactForm, self).updateWidgets()
        urltool = plone_api.portal.get_tool(name='portal_url')
        portal = urltool.getPortalObject()
        subject = u'{portal_title}: {title}'.format(
            portal_title=portal.getProperty('title').decode('utf-8'),
            title=self.development.title.value,
        )
        self.widgets['subject'].mode = HIDDEN_MODE
        self.widgets['subject'].value = subject

    @property
    def already_sent(self):
        return self._email_sent

    @button.buttonAndHandler(PMF(u'label_send', default='Send'), name='send')
    def handle_send(self, action):
        """Send button for sending the email."""
        can_send = True
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        if 'captcha' in data:
            can_send = False
            # Verify the user input against the captcha
            captcha = CaptchaValidator(
                self.context,
                self.request,
                None,
                IContactForm['captcha'],
                None,
            )
            try:
                can_send = captcha.validate(data['captcha'])
            except WrongCaptchaCode, e:
                self.status = e.doc()
                return

        if not self.already_sent and can_send:
            if self.send_email(data):
                self._email_sent = True
                plone_api.portal.show_message(
                    message=_(u'Your contact request was sent successfully.'),
                    request=self.request,
                    type='info',
                )
        self.request.response.redirect(self.request.URL)
        return

    def send_email(self, data):
        """Construct and send an email message."""
        urltool = plone_api.portal.get_tool(name='portal_url')
        portal = urltool.getPortalObject()

        try:
            email_charset = plone_api.portal.get_registry_record(
                'plone.email_charset',
            )
        except plone_api.exc.InvalidParameterError:
            email_charset = portal.getProperty('email_charset', 'utf-8')

        recipient = None
        portal_address = portal.getProperty('email_from_address')

        try:
            agent = self.item_info
            recipient = agent.email.value
        except Exception:
            recipient = portal_address

        agent_contact_info = u''
        if self.email_override is not None:
            recipient = self.email_override

            if agent is not None:
                agent_name = getattr(agent, 'name')
                if agent_name is not None:
                    agent_name = agent_name.value
                agent_email = getattr(agent, 'email')
                if agent_email is not None:
                    agent_email = agent_email.value
                agent_contact_info = translate(
                    EMAIL_TEMPLATE_AGENT,
                    context=self.request,
                ).format(
                    agent_name=agent_name,
                    agent_email=agent_email,
                )

        recipients = [recipient]
        bcc = self.config.get('contact_form_bcc', None)
        if bcc is not None:
            recipients += [formataddr(addr) for addr in getaddresses((bcc, ))]

        sender = formataddr((data['name'], data['sender_from_address']))

        subject = data['subject']
        data['url'] = self.request.getURL()
        if data['phone'] is None:
            data['phone'] = u''
        body = translate(
            EMAIL_TEMPLATE,
            context=self.request,
        ).format(**data)
        body = u'\n'.join([body, agent_contact_info])
        email_msg = message_from_string(body.encode(email_charset))
        email_msg['To'] = formataddr((recipient, recipient))

        plone_api.portal.send_email(
            sender=sender,
            recipient=recipients,
            subject=subject,
            body=email_msg,
        )
        return True


# Register Captcha validator for the captcha field in the ICaptchaForm
validator.WidgetValidatorDiscriminators(
    CaptchaValidator, field=IContactForm['captcha'])


@implementer(IDevelopmentDetails)
class DevelopmentDetails(BrowserView):
    """Detail view for MLS developments."""

    _item = None
    _contact_form = None
    _contact_info = None

    if PLONE_5:
        index = ViewPageTemplateFile('templates/development_details_p5.pt')
    elif PLONE_4:
        index = ViewPageTemplateFile('templates/development_details_view.pt')

    def __call__(self):
        self.setup()
        return self.render()

    def render(self):
        return self.index()

    def setup(self):
        self.registry = getUtility(IRegistry)  # noqa
        if PLONE_5:
            from Products.CMFPlone.resources import add_resource_on_request
            if self.use_fotorama():
                add_resource_on_request(self.request, 'psplonefotorama')
            try:
                pkg_resources.get_distribution('ps.plone.realestatefont')
            except pkg_resources.DistributionNotFound:
                pass
            else:
                from Products.GenericSetup.tool import UNKNOWN
                setup = plone_api.portal.get_tool(name='portal_setup')
                profile = 'profile-ps.plone.realestatefont:default'
                if setup.getLastVersionForProfile(profile) != UNKNOWN:
                    add_resource_on_request(
                        self.request,
                        'psplonerealestatefont',
                    )

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_DEVELOPMENT_COLLECTION, {})

    @property
    def item(self):
        if self._item is None:
            self._item = self._get_item()
        return self._item

    def _get_item(self):
        cache = IAnnotations(self.request)
        item = cache.get('ps.plone.mls.development.traversed', None)

        if item is None:
            item_id = getattr(self.request, 'development_id', None)
            if not item_id:
                return

            portal_state = queryMultiAdapter(
                (self.context, self.request),
                name='plone_portal_state',
            )
            lang = portal_state.language()
            item = api.get_development(
                item_id=item_id,
                context=self.context,
                request=self.request,
                lang=lang,
            )
        return item

    @property
    def lead_image(self):
        """"""
        item = self.item
        try:
            img = item.lead_image
        except AttributeError:
            pass
        else:
            return img

        try:
            images = item.pictures()
        except AttributeError:
            pass
        else:
            if len(images) > 0:
                return images[0]

    @property
    def map_id(self):
        """Generate a unique css id for the map."""
        try:
            item_id = self.item.id.value
        except Exception:
            item_id = 'unknown'
        return u'map__{0}'.format(item_id)

    def javascript_map(self):
        """Return the JS code for the map."""
        if not hasattr(self.item, 'geolocation') or not self.item.geolocation:  # noqa
            return

        icon = getattr(self.item, 'icon', None)
        if icon is not None:
            icon = icon.value
        icon_url = json.dumps(icon)

        lat, lng = self.item.geolocation.value.split(',')

        return MAP_JS.format(
            icon=icon_url,
            lat=lat,
            lng=lng,
            map_id=self.map_id,
            zoom=self.config.get('map_zoom_level', 7),
            ak=self.googleapi,
        )

    @property
    def googleapi(self):
        if not HAS_UI_SETTINGS:
            return ''

        if self.registry is not None:
            try:
                settings = self.registry.forInterface(IMLSUISettings)  # noqa
            except Exception:
                logger.warning('MLS UI settings not available.')
            else:
                return getattr(settings, 'googleapi', '')
        return ''

    def live_chat_embedding(self):
        """Return embedding code for live chat widget if it is enabled."""
        if not self.config.get('enable_live_chat', False):
            return
        embedding_code = getattr(self.item, 'live_chat_embedding', None)
        if embedding_code is not None:
            embedding_code = embedding_code.value
        return embedding_code

    def use_fotorama(self):
        if not HAS_UI_SETTINGS:
            return False

        if self.registry is not None:
            try:
                settings = self.registry.forInterface(IMLSUISettings)  # noqa
            except Exception:
                logger.warning('MLS UI settings not available.')
            else:
                return getattr(settings, 'slideshow') == u'fotorama'
        return False

    def use_galleria(self):
        if not HAS_UI_SETTINGS:
            return True

        if self.registry is not None:
            try:
                settings = self.registry.forInterface(IMLSUISettings)  # noqa
            except Exception:
                logger.warning('MLS UI settings not available.')
            else:
                return getattr(settings, 'slideshow') == u'galleria'
        # Fallback: 'galleria' is the default.
        return True

    def titles_for_phases(self):
        """Get the titles for the Development Phase fields."""
        raw = api.DevelopmentPhase.get_field_titles(self.item._api)
        return raw.get('response', {}).get('fields', {})

    def titles_for_groups(self):
        """Get the titles for the Property Group fields."""
        raw = api.PropertyGroup.get_field_titles(self.item._api)
        return raw.get('response', {}).get('fields', {})

    def distance_class(self):
        """Count how many distances are set"""
        counter = 0
        # get all distances
        item = self.item
        airport = item.airport_name or item.airport_distance
        bank = item.bank_name or item.bank_distance
        gas_station = item.gas_station_name or item.gas_station_distance
        hospital = item.hospital_name or item.hospital_distance
        shopping = item.shopping_name or item.shopping_distance

        if airport is not None:
            counter += 1
        if bank is not None:
            counter += 1
        if gas_station is not None:
            counter += 1
        if hospital is not None:
            counter += 1
        if shopping is not None:
            counter += 1

        return 'count_{0}'.format(counter)

    def show_section_contact(self):
        """Should the contact us section be shown at all?"""
        show_form = self.contact_form() is not None
        show_info = self.contact_info() is not None
        return show_info or show_form

    def _get_contact_info(self):
        if self._contact_info is not None:
            return self._contact_info

        item = self.item
        agency = getattr(item, 'agency', None)
        if agency is not None and agency() is not None:
            agency = agency()
            agency.override(context=self.context)
        agent = getattr(item, 'agent', None)
        if agent is not None and agent() is not None:
            agent = agent()
            agent.override(context=self.context)
        self._contact_info = {
            'agency': agency,
            'agent': agent,
        }
        return self._contact_info

    def contact_info(self):
        """Get the contact information, if enabled."""
        if not self.config.get('show_contact_info', False):
            return
        return self._get_contact_info()

    def contact_form(self):
        """Get the contact form, if enabled."""
        if not self.config.get('show_contact_form', False):
            return

        if self._contact_form is not None:
            return self._contact_form

        item_info = self._get_contact_info().get('agent')
        z2.switch_on(self, request_layer=IFormLayer)
        self._contact_form = ContactForm(
            aq_inner(self.context),
            self.request,
            info=item_info,
            development=self.item,
        )
        if HAS_WRAPPED_FORM:
            alsoProvides(self._contact_form, IWrappedForm)
        return self._contact_form

    def contact_link(self):
        return self.config.get('show_contact_link', False)

    def get_field_label(self, field_name):
        """Get the field label for ``field_name``.

        Do this even if the data may not exist within the current
        development object.
        """
        field = api.Field(field_name, None, self.item)
        return field.title

    def format_distance(self, name, distance):
        """Format the distance labels.

        Do this in the form of ``name - distance`` but also correctly handle
        None values.
        """
        text = []
        if name is not None:
            text.append(name)
        if distance is not None:
            text.append(distance)
        return u' - '.join(text)

    def base_url(self):
        return self.context.absolute_url()

    def group_listings(self, group=None):
        """Return the property group listings."""
        try:
            group_id = group.id.value
        except Exception:
            return

        try:
            item = api.PropertyGroup.get(self.item._api, group_id)
        except exceptions.ResourceNotFound:
            return None

        params = {
            'sort_on': 'last_activated_date',
            'reverse': '1',
        }
        try:
            results, batching = item.listings(params=params)
        except exceptions.MLSError, e:
            logger.warn(e)

        if results is None:
            return
        return ListingBatch(results, 0, batch_data=batching)


class HeaderViewlet(ViewletBase):
    """Header Image"""

    _id = None
    _title = None
    _headline = None
    _location = None
    _logo = None
    _banner = None
    _has_banner = False

    @property
    def available(self):
        # do we have a development?
        self._id = getattr(self.request, 'development_id', None)
        if self._id is not None:
            return True
        else:
            return False

    @property
    def get_title(self):
        """Get development title"""
        return self._title

    @property
    def get_headline(self):
        """Get development headline"""
        return self._headline

    @property
    def get_location(self):
        """Get development location"""
        return self._location

    @property
    def get_logo(self):
        """Get development logo"""
        return self._logo

    @property
    def get_banner(self):
        """Get development header"""
        return self._banner

    def update(self):
        """Prepare view related data."""
        super(HeaderViewlet, self).update()

        if self.available:
            self._set_development_info()

    def _set_banner(self, item):
        """Look for available Header image"""
        try:
            # banner image as regular data
            self._banner = item.banner_image.value
            self._has_banner = True
        except Exception:
            self._has_banner = False

    def _set_development_info(self):
        """Set all available data for the development header"""
        cache = IAnnotations(self.request)
        item = cache.get('ps.plone.mls.development.traversed', None)

        if item is not None:
            # try to set the available data
            try:
                self._logo = item.logo.value
            except Exception:
                pass
            try:
                self._title = item.title.value
            except Exception:
                pass
            try:
                self._headline = item.headline.value
            except Exception:
                pass
            try:
                seq = (item.city.value,
                       item.subdivision.value,
                       item.country.value)
                joint = ', '
                self._location = joint.join(seq)
            except Exception:
                self._location = item.location.value
            # set header image
            self._set_banner(item)


class DevelopmentCanonicalURL(ViewletBase):
    """Defines a canonical link relation viewlet.

    This needs to be displayed across the site. A canonical page is
    the preferred version of a set of pages with highly similar
    content. For more information, see:
    https://tools.ietf.org/html/rfc6596
    https://support.google.com/webmasters/answer/139394?hl=en
    """

    @memoize
    def render(self):
        context_state = queryMultiAdapter(
            (self.context, self.request), name=u'plone_context_state')
        base_url = context_state.current_base_url()
        return u'    <link rel="canonical" href="{0}" />'.format(base_url)



# -*- coding: utf-8 -*-
"""Migration steps for ps.plone.mls."""

from plone import api
from ps.plone.mls import config


def migrate_to_1001(context):
    """Migrate from 1000 to 1001.

    * Activate portal actions.
    """
    setup = api.portal.get_tool(name='portal_setup')
    setup.runImportStepFromProfile(config.INSTALL_PROFILE, 'actions')


def migrate_to_1002(context):
    """Migrate from 1001 to 1002.

    * ensure mls.css
    """
    setup = api.portal.get_tool(name='portal_setup')
    setup.runImportStepFromProfile(config.INSTALL_PROFILE, 'cssregistry')
    setup.runImportStepFromProfile(config.INSTALL_PROFILE, 'jsregistry')


def migrate_css(context):
    """Migration step for simple css updates."""
    setup = api.portal.get_tool(name='portal_setup')
    setup.runImportStepFromProfile(config.INSTALL_PROFILE, 'cssregistry')


def migrate_to_1005(context):
    """Migrate from 1004 to 1005

    * add ps.plone.realestatefont
    """
    quickinstaller = api.portal.get_tool(name='portal_quickinstaller')

    # Add ps.fonts.iconmagic
    if not quickinstaller.isProductInstalled('ps.plone.realestatefont'):
        quickinstaller.installProduct('ps.plone.realestatefont')


def migrate_to_1007(context):
    """Migrate from 1006 to 1007.

    * Hide default DublinCoreViewlet
    """
    setup = api.portal.get_tool(name='portal_setup')
    setup.runImportStepFromProfile(config.INSTALL_PROFILE, 'viewlets')


def migrate_to_1008(context):
    """"Migrate from 1007 to 1008

    * Update JS registry.
    """
    try:
        js = api.portal.get_tool(name='portal_javascripts')
    except AttributeError:
        pass
    else:
        maps_url = 'http://maps.google.com/maps/api/js?sensor=false'
        js.unregisterResource(maps_url)
        maps_ssl = 'https://maps-api-ssl.google.com/maps/api/js?sensor=false'
        js.unregisterResource(maps_ssl)


def migrate_to_1009(context):
    """"Migrate from 1008 to 1009.

    * Update portal actions.
    """
    setup = api.portal.get_tool(name='portal_setup')
    setup.runImportStepFromProfile(config.INSTALL_PROFILE, 'actions')



# -*- coding: utf-8 -*-
"""Show Listings for developments."""

from mls.apiclient import exceptions
from plone.memoize.view import memoize
from plone.mls.core.navigation import ListingBatch
from plone.mls.listing.browser.interfaces import IBaseListingItems
from Products.Five import BrowserView
from ps.plone.mls import api
from ps.plone.mls import config
from ps.plone.mls import logger
from ps.plone.mls.interfaces import IDevelopmentListings
from zope.annotation.interfaces import IAnnotations
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.traversing.browser.absoluteurl import absoluteURL


@implementer(IBaseListingItems, IDevelopmentListings)
class DevelopmentListings(BrowserView):
    """Show listings for a specific development."""

    _listings = None
    _batching = None
    limit = None
    item = None

    def __init__(self, context, request):
        super(DevelopmentListings, self).__init__(context, request)
        self.update()

    def available(self):
        return self.item is not None

    def update(self):
        """Prepare view related data."""
        self.context_state = queryMultiAdapter(
            (self.context, self.request),
            name='plone_context_state',
        )
        self.limit = self.config.get('limit_listings', 25)
        self._get_item()
        self._get_listings()

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_DEVELOPMENT_COLLECTION, {})

    @property
    def batching(self):
        return ListingBatch(
            self.listings,
            self.limit,
            self.request.get('b_start', 0),
            orphan=1,
            batch_data=self._batching,
        )

    @property
    def listings(self):
        """Return listing results."""
        return self._listings

    def _get_item(self):
        cache = IAnnotations(self.request)
        self.item = cache.get('ps.plone.mls.development.traversed', None)

    def _get_listings(self):
        """Get the listings for the development.

        Optional filter by development phase or property group via GET
        params from the request.
        """
        item = self.item
        if item is None:
            return

        params = {
            'limit': self.limit,
            'offset': self.request.get('b_start', 0),
            'sort_on': 'last_activated_date',
            'reverse': '1',
        }

        phase_id = self.request.form.get('phase')
        group_id = self.request.form.get('group')

        if phase_id:
            try:
                item = api.DevelopmentPhase.get(self.item._api, phase_id)
            except exceptions.ResourceNotFound:
                item = None
        elif group_id:
            try:
                item = api.PropertyGroup.get(self.item._api, group_id)
            except exceptions.ResourceNotFound:
                item = None

        try:
            results, batching = item.listings(params=params)
        except exceptions.MLSError, e:
            logger.warn(e)
        else:
            self._listings = results
            self._batching = batching

    @memoize
    def view_url(self):
        """Generate view url."""
        try:
            item_id = self.item.id.value
        except Exception:
            item_id = ''

        return '/'.join([
            absoluteURL(self.context, self.request),
            item_id,
            '',
        ])



# -*- coding: utf-8 -*-
"""Post install import steps for ps.plone.mls."""

from plone import api
from Products.CMFPlone.interfaces import INonInstallable
from Products.GenericSetup.interfaces import IProfileImportedEvent
from ps.plone.mls import config
from zope.component import adapter
from zope.interface import implementer

import pkg_resources


ADD_ONS = [
    'ps.plone.fotorama',
]


@implementer(INonInstallable)
class HiddenProfiles(object):
    """Define hidden GenericSetup profiles."""

    def getNonInstallableProducts(self):
        return [
            'ps.plone.mls.upgrades',
        ]

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        return [
            'ps.plone.mls:testfixture',
            'ps.plone.mls:uninstall',
        ]


@adapter(IProfileImportedEvent)
def handle_profile_imported_event(event):
    """Update 'last version for profile' after a full import."""
    qi = api.portal.get_tool(name='portal_quickinstaller')
    if not qi.isProductInstalled(config.PROJECT_NAME):
        return

    if event.profile_id == 'profile-plone.app.upgrade.v50:to50alpha3':
        setup = api.portal.get_tool(name='portal_setup')
        setup.runAllImportStepsFromProfile(config.INSTALL_PROFILE)


def install_add_ons(context):
    """Install additional available add-ons."""
    if not context.readDataFile('plone.mls.listing_various.txt'):
        return

    setup = api.portal.get_tool(name='portal_setup')
    quickinstaller = api.portal.get_tool(name='portal_quickinstaller')
    for item in ADD_ONS:
        try:
            pkg_resources.get_distribution(item)
        except pkg_resources.DistributionNotFound:
            continue
        # Only install add-ons which are not installed yet.
        if not quickinstaller.isProductInstalled(item):
            if quickinstaller.isProductInstallable(item):
                try:
                    profile = 'profile-{0}:default'.format(item)
                    setup.runAllImportStepsFromProfile(profile)
                except AttributeError:
                    pass



# -*- coding: utf-8 -*-
"""MLS integration api."""

from mls.apiclient import api
from mls.apiclient import exceptions
from mls.apiclient import resources
from plone import api as plone_api
from plone.memoize import ram
from plone.mls.core.api import get_settings
from plone.mls.listing.api import get_agency_info
from ps.plone.mls import _
from ps.plone.mls import utils
from time import time


MIN_MAX_FIELDS = [
    'baths',
    'beds',
    'lot_size',
    'interior_area',
]


def api_cachekey(fun, self, *args, **kwargs):
    """Create cache key for plone.memoize."""
    try:
        api_ = args[0]
    except IndexError:
        try:
            api_ = self._api
        except AttributeError:
            raise ram.DontCache
    if not isinstance(api_, api.API):
        raise ram.DontCache

    try:
        item_id = self.get_id()
    except TypeError:
        item_id = '-'

    args_ = '__'.join(
        [value for value in args if not isinstance(value, api.API)],
    )

    kwargs_ = '__'.join(
        ['{0}_{1}'.format(key, value) for key, value in kwargs.iteritems()],
    )

    timeout = plone_api.portal.get_registry_record(
        name='ps.plone.mls.interfaces.IMLSCachingSettings.timeout',
        default=3600,
    )
    key = u'{0}_{1}_{2}_{3}_{4}_{5}_{6}_{7}_{8}'.format(
        self.endpoint,
        item_id,
        fun.func_name,
        args_,
        kwargs_,
        api_.base_url,
        api_.lang,
        api_.api_key,
        time() // timeout,
    )
    return key


def _remove_omitted(params, omit):
    """Removed omitted keys from the params dict."""
    if omit is not None and isinstance(omit, (list, tuple)):
        for item in omit:
            try:
                del params[item]
            except KeyError:
                continue


def prepare_search_params(params, context=None, omit=None):
    """Prepare search params."""
    settings = get_settings(context=context)
    result = {}

    _remove_omitted(params, omit)

    for item in params:
        if item in MIN_MAX_FIELDS:
            min_max = params[item]
            if isinstance(min_max, (list, tuple)):
                if len(min_max) > 0 and min_max[0] != '--MINVALUE--':
                    result[item + '_min'] = min_max[0]
                if len(min_max) > 1 and min_max[1] != '--MAXVALUE--':
                    result[item + '_max'] = min_max[1]
                continue

        # Convert lists and tuples to comma separated lists.
        if isinstance(params[item], (list, tuple)):
            if len(params.get(item, ())) > 0:
                params[item] = ','.join(params[item])
            else:
                params[item] = None

        # Remove all None-Type values.
        if params[item] is not None:
            value = params[item]
            if isinstance(value, unicode):
                value = value.encode('utf-8')
            result[item] = value

    agency_developments = result.pop('agency_developments', False)
    if agency_developments is True:
        result['agency_developments'] = settings.get('agency_id', None)

    return result


def get_api(context=None, lang=None):
    """Get the API Client based on the local configuration."""
    settings = get_settings(context=context)
    base_url = settings.get('mls_site', None)
    api_key = settings.get('mls_key', None)
    debug = plone_api.env.debug_mode
    mls = api.API(base_url, api_key=api_key, lang=lang, debug=debug)
    return mls


def get_development(item_id=None, context=None, request=None, lang=None):
    """Get a single development."""
    mlsapi = get_api(context=context, lang=lang)
    # load from cache if possible
    try:
        item = Development.get(mlsapi, item_id)
    except exceptions.ServerError:
        if not plone_api.user.is_anonymous():
            message = _(
                u'An error occured trying to connect to the '
                u'configured MLS. Please check your settings or try '
                u'again later.',
            )
            plone_api.portal.show_message(
                message=message,
                request=request,
                type='warning',
            )
    except exceptions.ResourceNotFound:
        message = _(
            u'The requested development was not found.',
        )
        plone_api.portal.show_message(
            message=message,
            request=request,
            type='info',
        )
    except exceptions.ConnectionError:
        pass
    else:
        return item


class Field(object):
    """Wraps the api data into a field structure."""

    def __init__(self, name, value, resource, title=None):
        self.name = name
        self.value = value
        self.title = title

        if self.title is not None:
            return

        # Try to get the correct label for the field.
        titles = resource.get_field_titles(resource._api).get('response', {})

        try:
            self.title = titles['fields'][name]
        except (KeyError, TypeError):
            pass
        else:
            return

        group_fields = titles.get('group_fields', [])
        for group in group_fields:
            try:
                self.title = group_fields[group][name]
            except (KeyError, TypeError):
                continue
            else:
                break
        if self.title is None:
            self.title = name


class CacheMixin(object):
    """Extend API resources to handle some caching."""

    @classmethod
    @ram.cache(api_cachekey)
    def get(cls, api, resource_id):
        return super(CacheMixin, cls).get(api, resource_id)

    @classmethod
    @ram.cache(api_cachekey)
    def get_field_order(cls, api):
        return super(CacheMixin, cls).get_field_order(api)

    @classmethod
    @ram.cache(api_cachekey)
    def get_field_titles(cls, api):
        return super(CacheMixin, cls).get_field_titles(api)

    @classmethod
    @ram.cache(api_cachekey)
    def search(cls, api, params=None):
        return super(CacheMixin, cls).search(api, params=params)

    def __getattr__(self, name):
        """Return a data attribute or raises AttributeError.

        This version wraps the return value into a Field class for better
        access in Plone.
        """
        try:
            value = self._data[name]
        except KeyError:
            return object.__getattribute__(self, name)
        else:
            if value is not None:
                return Field(name, value, self)


class Agency(CacheMixin, resources.Agency):
    """'Agency' entity resource class with caching support."""

    def override(self, context=None):
        """Override the agency information with local settings."""
        settings = get_agency_info(context=context)
        if settings is None:
            return

        mls_settings = get_settings(context=context)
        agency_id = mls_settings.get('agency_id', None)

        if self._data.get('id', None) == agency_id:
            if settings and settings.get('force', False) is True:
                pass
            else:
                return

        mapping = {
            'agency_address': 'address',
            'agency_description': 'description',
            'agency_email': 'email',
            'agency_email_alternative': 'email_alternative',
            'agency_geo_location': 'geo_location',
            'agency_logo_url': 'logo',
            'agency_name': 'title',
            'agency_office_fax': 'office_fax',
            'agency_office_phone': 'office_phone',
            'agency_office_phone_alternative': 'office_phone_alternative',
            'agency_website': 'website',
        }

        utils.merge_local_contact_info(
            settings=settings,
            mapping=mapping,
            data=self._data,
        )


class Agent(CacheMixin, resources.Agent):
    """'Agent' entity resource class with caching support."""

    def override(self, context=None):
        """Override the agent information with local settings."""
        settings = get_agency_info(context=context)
        if settings is None:
            return

        mls_settings = get_settings(context=context)
        agency_id = mls_settings.get('agency_id', None)

        if self._data.get('id', None) == agency_id:
            if settings and settings.get('force', False) is True:
                pass
            else:
                return

        mapping = {
            'agent_avatar_url': 'avatar',
            'agent_cell_phone': 'cell_phone',
            'agent_email': 'email',
            'agent_fax': 'fax',
            'agent_name': 'name',
            'agent_office_phone': 'office_phone',
            'agent_title': 'user_title',
        }

        utils.merge_local_contact_info(
            settings=settings,
            mapping=mapping,
            data=self._data,
        )


class Development(CacheMixin, resources.Development):
    """'Development Project' entity resource class with caching support."""

    def __init__(self, api, data):
        super(Development, self).__init__(api, data)
        self.__class_agency__ = Agency
        self.__class_agent__ = Agent
        self.__class_group__ = PropertyGroup
        self.__class_phase__ = DevelopmentPhase

    @ram.cache(api_cachekey)
    def groups(self, params=None):
        """Search for property groups within that development."""
        return super(Development, self).groups(params=params)

    @ram.cache(api_cachekey)
    def phases(self, params=None):
        """Search for development phases within that development."""
        return super(Development, self).phases(params=params)

    @ram.cache(api_cachekey)
    def listings(self, params=None):
        """Search for listings assigned to that development project."""
        return super(Development, self).listings(params=params)


class DevelopmentPhase(CacheMixin, resources.DevelopmentPhase):
    """'Development Phase' entity resource class with caching support."""

    @ram.cache(api_cachekey)
    def listings(self, params=None):
        """Search for listings assigned to that development phase."""
        return super(DevelopmentPhase, self).listings(params=params)


class Listing(CacheMixin, resources.Listing):
    """'Listing' entity resource class with caching support."""


class PropertyGroup(CacheMixin, resources.PropertyGroup):
    """'Property Group' entity resource class with caching support."""

    @ram.cache(api_cachekey)
    def listings(self, params=None):
        """Search for listings assigned to that property group."""
        return super(PropertyGroup, self).listings(params=params)



# -*- coding: utf-8 -*-
"""Module with all imports used for the ExternalMethod's.

Then create external methods in ZMI for get_item, get_children,
get_catalog_results and export_content as described in the installation docs.
"""
from collective.jsonify import get_catalog_results  # noqa: F401
from collective.jsonify import get_children  # noqa: F401
from collective.jsonify.export import export_content as export_content_orig
from ps.plone.mls.browser.jsonify import get_item  # noqa: F401
from ps.plone.mls.browser.jsonify import MLSWrapper


def export_content(self):
    return export_content_orig(
        self,
        # absolute path to directory for the JSON export
        basedir='/tmp',
        # optional callback. Returns True to skip an item.
        skip_callback=lambda item: False,
        # optional list of classnames to skip
        extra_skip_classname=[],
        custom_wrapper=MLSWrapper,
        # batch_start=0,
        # batch_size=5000,
        # batch_previous_path='/absolute/path/to/last/exported/item'
    )



# -*- coding: utf-8 -*-
"""MLS development collection."""

from plone.app.layout.viewlets.common import ViewletBase
from plone.directives import form
from plone.memoize.view import memoize
from plone.mls.core.navigation import ListingBatch
from plone.mls.listing.browser.interfaces import IBaseListingItems
from plone.mls.listing.browser.interfaces import IListingDetails
from plone.supermodel.directives import fieldset
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ps.plone.mls import _
from ps.plone.mls import api
from ps.plone.mls import config
from ps.plone.mls import PLONE_4
from ps.plone.mls import PLONE_5
from ps.plone.mls.interfaces import IDevelopmentCollection
from ps.plone.mls.interfaces import IDevelopmentDetails
from ps.plone.mls.interfaces import IPossibleDevelopmentCollection
from z3c.form import button
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.interface import noLongerProvides
from zope.traversing.browser.absoluteurl import absoluteURL

import copy


#: Reduce the API load by defining the fields we need.
FIELDS = [
    'id',
    'title',
    'description',
    'logo',
    'location',
    'lot_size',
    'location_type',
    'number_of_listings',
    'number_of_groups',
]

EXCLUDED_SEARCH_FIELDS = [
    'map_zoom_level',
    'show_banner_image',
    'show_contact_info',
    'show_contact_form',
    'contact_override',
    'contact_form_bcc',
    'show_contact_link',
    'show_captcha',
    'enable_live_chat',
    'modify_url',
    'limit_developments',
    'limit_listings',
]


class DevelopmentCollectionViewlet(ViewletBase):
    """Dynamic collection of MLS developments."""

    _items = None
    _batching = None
    limit = None

    if PLONE_5:
        index = ViewPageTemplateFile('templates/development_results_p5.pt')
    elif PLONE_4:
        index = ViewPageTemplateFile('templates/development_results.pt')

    @property
    def available(self):
        """Check if this viewlet is available for rendering."""
        return IDevelopmentCollection.providedBy(self.context) and \
            not IDevelopmentDetails.providedBy(self.view) and \
            not IBaseListingItems.providedBy(self.view) and \
            not IListingDetails.providedBy(self.view)

    @property
    def config(self):
        """Get the embedding configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(config.SETTINGS_DEVELOPMENT_COLLECTION, {})

    def update(self):
        """Prepare view related data."""
        super(DevelopmentCollectionViewlet, self).update()
        self.context_state = queryMultiAdapter(
            (self.context, self.request), name='plone_context_state',
        )

        if self.available:
            # Only query the MLS if we show the viewlet.
            self.limit = self.config.get('limit_developments', 5)
            self._get_items()

    @property
    @memoize
    def items(self):
        """Return the collection items."""
        return self._items

    def _get_items(self):
        """Get the collection items from the MLS."""
        lang = self.portal_state.language()
        mlsapi = api.get_api(context=self.context, lang=lang)
        fields = copy.copy(FIELDS)
        if self.config.get('show_banner_image', False):
            fields.append('banner_image')
        params = {
            # 'summary': '1',
            'fields': u','.join(fields),
            'limit': self.limit,
            'offset': self.request.get('b_start', 0),
        }
        params.update(self.config)
        params = api.prepare_search_params(
            params,
            context=self.context,
            omit=EXCLUDED_SEARCH_FIELDS,
        )
        try:
            result = api.Development.search(mlsapi, params=params)
        except Exception:
            pass
        else:
            self._items = result.get_items()
            headers = result.get_headers()
            self._batching = {
                'results': headers.get('CountTotal'),
            }

    @memoize
    def view_url(self):
        """Generate view url."""
        if not self.context_state.is_view_template():
            # We have the default view rendered.
            return self.context_state.current_base_url()
        else:
            return '/'.join([
                absoluteURL(self.context, self.request),
                '',
            ])

    @property
    def batching(self):
        """Create the batch provider for the collection items."""
        if not self.items:
            return
        return ListingBatch(
            self.items,
            self.limit,
            self.request.get('b_start', 0),
            orphan=1,
            batch_data=self._batching,
        )

    def get_url(self, item):
        """Get the (possibly modified) URL for the development item."""
        url = u'{0}{1}'.format(self.view_url(), item.id.value)
        if self.config.get('modify_url', True):
            url = u'{0}___{1}-{2}'.format(
                url,
                item.title.value,
                item.location.value,
            )
        return url


FIELDS_FILTER = (
    'agency_developments',
    'q',
    'sort_on',
    'reverse',
)


class IDevelopmentCollectionConfiguration(form.Schema):
    """Development Collection Configuration Form."""

    fieldset(
        'filter',
        label=_(u'Filter Options'),
        fields=FIELDS_FILTER,
    )

    # Fields for 'Defautl' fieldset.
    map_zoom_level = schema.Int(
        default=7,
        description=_(
            u'Set the resolution for the map. 0 is the lowest zoom level, in '
            u'which the entire world can be seen. The higher the zoom level, '
            u'the more details can be seen. The available maximum zoom level '
            u'value differs from area to area and can change over time, also '
            u'depending on the used map provider.',
        ),
        min=0,
        max=21,
        required=True,
        title=_(u'Zoom level for maps'),
    )

    show_banner_image = schema.Bool(
        default=False,
        description=_(
            u'If enabled, show the development banner in the collection '
            u'results.',
        ),
        required=False,
        title=_(u'Show Banner Image'),
    )

    show_contact_info = schema.Bool(
        default=False,
        description=_(
            u'If enabled, the contact information for a development will be '
            u'shown on the detail page for a development.',
        ),
        required=False,
        title=_(u'Show Contact Information'),
    )

    show_contact_form = schema.Bool(
        default=False,
        description=_(
            u'If enabled, a form to contact the responsible agent will be '
            u'shown on the detail page for a development.',
        ),
        required=False,
        title=_(u'Show Contact Form'),
    )

    contact_override = schema.TextLine(
        description=_(
            u'Send the contact form to this email instead direct to the agent',
        ),
        required=False,
        title=_(u'Alternative Email recipient'),
    )

    contact_form_bcc = schema.TextLine(
        description=_(
            u'E-mail addresses which receive a blind carbon copy (comma '
            u'separated).',
        ),
        required=False,
        title=_(u'BCC Recipients'),
    )

    show_contact_link = schema.Bool(
        default=False,
        description=_(
            u'If the contact form is enabled, an anchor link will show as '
            u'quick navigation to the form.',
        ),
        required=False,
        title=_(u'Show Contact-Us anchor link'),
    )

    show_captcha = schema.Bool(
        default=False,
        description=_(
            u'Show a captcha field within the contact form to prevent spam.',
        ),
        required=False,
        title=_(u'Show Captcha'),
    )

    enable_live_chat = schema.Bool(
        default=False,
        description=_(
            u'Enable live chat embedding for developments with embedding '
            u'code.',
        ),
        required=False,
        title=_(u'Enable Live Chat Embedding'),
    )

    modify_url = schema.Bool(
        default=True,
        description=_(
            u'Modify the individual development URLs to show extra '
            u'information, such as the Title and Location',
        ),
        required=False,
        title=_(u'Modify URLs'),
    )

    limit_developments = schema.Int(
        default=5,
        description=_(
            u'How many developments should be shown per page?',
        ),
        required=True,
        title=_(u'Developments per Page'),
    )

    limit_listings = schema.Int(
        default=25,
        description=_(
            u'How many listings should be shown per devlopment listings page?',
        ),
        required=True,
        title=_(u'Listings per Page'),
    )

    # Fields for 'Filter Options' fieldset.
    agency_developments = schema.Bool(
        default=False,
        description=_(
            u'If enabled, only developments for the configured agency '
            u'will be shown.',
        ),
        required=False,
        title=_(u'Agency Developments'),
    )

    q = schema.TextLine(
        description=_(
            u'Enter a search term to filter the results.',
        ),
        required=False,
        title=_(u'Searchable Text'),
    )

    sort_on = schema.Choice(
        description=_(
            u'How should the results be sorted? If nothing is selected, the '
            u'results are sorted by relevance.',
        ),
        required=False,
        title=_('Sort results by'),
        vocabulary='ps.plone.mls.developments.sort_options',
    )

    reverse = schema.Bool(
        default=False,
        required=False,
        title=_('Reverse sort order?'),
    )

    # location_state = schema.Choice(
    #     required=False,
    #     title=_(u'State'),
    #     source='plone.mls.listing.LocationStates',
    # )

    # location_county = schema.Choice(
    #     required=False,
    #     title=_(u'County'),
    #     source='plone.mls.listing.LocationCounties',
    # )

    # location_district = schema.Choice(
    #     required=False,
    #     title=_(u'District'),
    #     source='plone.mls.listing.LocationDistricts',
    # )

    # location_type = schema.Tuple(
    #     required=False,
    #     title=_(u'Location Type'),
    #     value_type=schema.Choice(
    #         source='plone.mls.listing.LocationTypes'
    #     ),
    # )

    # geographic_type = schema.Tuple(
    #     required=False,
    #     title=_(u'Geographic Type'),
    #     value_type=schema.Choice(
    #         source='plone.mls.listing.GeographicTypes'
    #     ),
    # )

    # view_type = schema.Tuple(
    #     required=False,
    #     title=_(u'View Type'),
    #     value_type=schema.Choice(
    #         source='plone.mls.listing.ViewTypes'
    #     ),
    # )


class DevelopmentCollectionConfiguration(form.SchemaForm):
    """Development Collection Configuration Form."""

    schema = IDevelopmentCollectionConfiguration
    label = _(u'\'Development Collection\' Configuration')
    description = _(
        u'Adjust the behaviour for this \'Development Collection\' viewlet.',
    )

    def getContent(self):
        annotations = IAnnotations(self.context)
        return annotations.get(
            config.SETTINGS_DEVELOPMENT_COLLECTION,
            annotations.setdefault(config.SETTINGS_DEVELOPMENT_COLLECTION, {}),
        )

    @button.buttonAndHandler(_(u'Save'))
    def handle_save(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        annotations = IAnnotations(self.context)
        annotations[config.SETTINGS_DEVELOPMENT_COLLECTION] = data
        self.request.response.redirect(absoluteURL(self.context, self.request))

    @button.buttonAndHandler(_(u'Cancel'))
    def handle_cancel(self, action):
        self.request.response.redirect(absoluteURL(self.context, self.request))


class DevelopmentCollectionStatus(object):
    """Return activation/deactivation status of the viewlet."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def can_activate(self):
        return IPossibleDevelopmentCollection.providedBy(self.context) and \
            not IDevelopmentCollection.providedBy(self.context)

    @property
    def active(self):
        return IDevelopmentCollection.providedBy(self.context)


class DevelopmentCollectionToggle(object):
    """Toggle DevelopmentCollection viewlet for the current context."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        msg_type = 'info'

        if IDevelopmentCollection.providedBy(self.context):
            # Deactivate DevelopmentCollection viewlet.
            noLongerProvides(self.context, IDevelopmentCollection)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u'\'Development Collection\' viewlet deactivated.')
        elif IPossibleDevelopmentCollection.providedBy(self.context):
            alsoProvides(self.context, IDevelopmentCollection)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u'\'Development Collection\' viewlet activated.')
        else:
            msg = _(
                u'The \'Development Collection\' viewlet does\'t work with '
                u'this content type. Add \'IPossibleDevelopmentCollection\' '
                u'to the provided interfaces to enable this feature.',
            )
            msg_type = 'error'

        self.context.plone_utils.addPortalMessage(msg, msg_type)
        self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-
"""Test Setup of ps.plone.mls."""

from plone import api
from plone.browserlayer.utils import registered_layers
from ps.plone.mls import PLONE_4
from ps.plone.mls.config import PROJECT_NAME
from ps.plone.mls.testing import INTEGRATION_TESTING


try:
    import unittest2 as unittest
except ImportError:
    import unittest


CSS = [
    '++resource++ps.plone.mls/mls.css',
]

JS = [
    '++resource++ps.plone.mls/mls.js',
]


class TestSetup(unittest.TestCase):
    """Validate setup process for ps.plone.mls."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']

    def test_product_is_installed(self):
        """Validate that our product is installed."""
        qi = self.portal.portal_quickinstaller
        self.assertTrue(qi.isProductInstalled(PROJECT_NAME))

    def test_addon_layer(self):
        """Validate that the browserlayer for our product is installed."""
        layers = [l.getName() for l in registered_layers()]
        self.assertIn('IPloneMLSLayer', layers)

    def test_cssregistry(self):
        """Validate the CSS file registration."""
        if not PLONE_4:
            return

        resource_ids = self.portal.portal_css.getResourceIds()
        for id in CSS:
            self.assertIn(id, resource_ids, '{0} not installed'.format(id))

    def test_jsregistry(self):
        """Validate the JS file registration."""
        if not PLONE_4:
            return

        resource_ids = self.portal.portal_javascripts.getResourceIds()
        for id in JS:
            self.assertIn(id, resource_ids, '{0} not installed'.format(id))

    def test_plone_app_dexterity_installed(self):
        """Validate that plone.app.dexterity is installed."""
        portal = self.layer['portal']
        qi = portal.portal_quickinstaller
        if qi.isProductAvailable('plone.app.dexterity'):
            self.assertTrue(qi.isProductInstalled('plone.app.dexterity'))
        else:
            self.assertTrue(
                'plone.app.dexterity' in qi.listInstallableProfiles(),
            )

    def test_plone_mls_listing_installed(self):
        """Validate that plone.mls.listing is installed."""
        qi = self.portal.portal_quickinstaller
        self.assertTrue(qi.isProductInstalled('plone.mls.listing'))


class UninstallTestCase(unittest.TestCase):
    """Validate uninstall process for ps.plone.mls."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']

        qi = self.portal.portal_quickinstaller
        with api.env.adopt_roles(['Manager']):
            qi.uninstallProducts(products=[PROJECT_NAME])

    def test_product_is_uninstalled(self):
        """Validate that our product is uninstalled."""
        qi = self.portal.portal_quickinstaller
        self.assertFalse(qi.isProductInstalled(PROJECT_NAME))

    def test_addon_layer_removed(self):
        """Validate that the browserlayer is removed."""
        layers = [l.getName() for l in registered_layers()]
        self.assertNotIn('IPloneMLSLayer', layers)

    def test_cssregistry(self):
        """Validate the CSS file unregistration."""
        if not PLONE_4:
            return

        resource_ids = self.portal.portal_css.getResourceIds()
        for id in CSS:
            self.assertNotIn(
                id, resource_ids,
                '{0} is still installed'.format(id),
            )

    def test_jsregistry(self):
        """Validate the JS file unregistration."""
        if not PLONE_4:
            return

        resource_ids = self.portal.portal_javascripts.getResourceIds()
        for id in JS:
            self.assertNotIn(
                id, resource_ids,
                '{0} is still installed'.format(id),
            )



# -*- coding: utf-8 -*-
"""Custom wrapper for collective.jsonify."""

from collective.jsonify.methods import _clean_dict
from collective.jsonify.methods import get_catalog_results  # noqa: F401
from collective.jsonify.methods import get_children  # noqa: F401
from collective.jsonify.wrapper import Wrapper
from plone.mls.core.interfaces import ILocalMLSSettings
from plone.mls.listing.browser.listing_collection import IListingCollection
from plone.mls.listing.browser.listing_search import IListingSearch
from plone.mls.listing.browser.localconfig import ILocalAgencyInfo
from plone.mls.listing.browser.recent_listings import IRecentListings
from ps.plone.mls import config
from ps.plone.mls.browser.listings.featured import IFeaturedListings
from ps.plone.mls.interfaces import IDevelopmentCollection
from ps.plone.mls.interfaces import IListingSearchBanner
from zope.annotation.interfaces import IAnnotations

import pprint
import sys
import traceback


try:
    import simplejson as json
except ImportError:
    import json

MLS_IFACE_MAPPING = {
    IDevelopmentCollection: config.SETTINGS_DEVELOPMENT_COLLECTION,
    IListingCollection: config.SETTINGS_LISTING_COLLECTION,
    IFeaturedListings: config.SETTINGS_LISTING_FEATURED,
    IRecentListings: config.SETTINGS_LISTING_RECENT,
    IListingSearch: config.SETTINGS_LISTING_SEARCH,
    IListingSearchBanner: config.SETTINGS_LISTING_SEARCH_BANNER,
    ILocalAgencyInfo: config.SETTINGS_LOCAL_AGENCY,
    ILocalMLSSettings: config.SETTINGS_LOCAL_MLS,
}


def get_item(self):
    """Get information about an item."""

    try:
        context_dict = MLSWrapper(self)
    except Exception, e:
        etype = sys.exc_info()[0]
        tb = pprint.pformat(traceback.format_tb(sys.exc_info()[2]))
        return 'ERROR: exception wrapping object: {0}: {1}\n{2}'.format(
            etype, str(e), tb,
        )

    passed = False
    while not passed:
        try:
            JSON = json.dumps(context_dict)
            passed = True
        except Exception, error:
            if 'serializable' in str(error):
                key, context_dict = _clean_dict(context_dict, error)
                pprint.pprint(
                    'Not serializable member {0} of {1} ignored'.format(
                        key, repr(self),
                    ),
                )
                passed = False
            else:
                return ('ERROR: Unknown error serializing object: {0}'.format(
                    error,
                ))
    self.REQUEST.response.setHeader('Content-type', 'application/json')
    return JSON


class MLSWrapper(Wrapper):
    """Wrapper for MLS embedding objects."""

    def __init__(self, context):
        super(MLSWrapper, self).__init__(context)
        self.get_mls_configuration()

    def get_mls_configuration(self):
        try:
            annotations = IAnnotations(self.context)
        except Exception:
            return

        self['_mls_config'] = {}

        config_items = MLS_IFACE_MAPPING.values()
        for item in config_items:
            settings = annotations.get(item, {})
            if settings:
                self['_mls_config'][item] = settings

        # get active viewlets/integrations
        active = []
        for (iface, config_key) in MLS_IFACE_MAPPING.items():
            if iface.providedBy(self.context):
                active.append(config_key)

        self['_mls_activated'] = active
